@@ -187,7 +187,8 @@ static void copy_full_link(Link **dest, Link *src)
  */
 static void build_digraph(analyze_context_t *actx, Parse_info pi)
 {
-	int i, link;
+	size_t link;
+	int i;
 	Priority p;
 	Link *lp;
 	List_o_links * lol;
@@ -891,7 +892,7 @@ static int strictly_smaller_name(const char * s, const char * t)
  */
 static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 {
-	int i;
+	size_t i;
 	const char * s;
 	Parse_info pi = sent->parse_info;
 	count_context_t * ctxt = sent->count_ctxt;
@@ -953,11 +954,11 @@ void free_analyze(Sentence s)
  */
 Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_pass)
 {
-	int i;
+	size_t i;
 	Linkage_info li;
 	DIS_node *d_root;
 	PP_node *pp;
-	Postprocessor *postprocessor;
+	Postprocessor *postprocessor = sent->dict->postprocessor;
 	Sublinkage *sublinkage;
 	Parse_info pi = sent->parse_info;
 	PP_node accum;			   /* for domain ancestry check */
@@ -966,7 +967,6 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 	analyze_context_t *actx = sent->analyze_ctxt;
 
 	sublinkage = x_create_sublinkage(pi);
-	postprocessor = sent->dict->postprocessor;
 	build_digraph(actx, pi);
 	actx->structure_violation = FALSE;
 	d_root = build_DIS_CON_tree(actx, pi); /* may set structure_violation to TRUE */
@@ -1049,12 +1049,12 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 
 		/* 'analyze_pass' logic added ALB 1/97 */
 		if (analyze_pass == PP_FIRST_PASS) {
-			post_process_scan_linkage(postprocessor,opts,sent,sublinkage);
+			post_process_scan_linkage(postprocessor, opts, sent, sublinkage);
 			if (!advance_DIS(d_root)) break;
 			else continue;
 		}
 
-		pp = do_post_process(postprocessor, opts, sent, sublinkage, TRUE);
+		pp = do_post_process(postprocessor, opts, sent, sublinkage, true);
 
 		if (pp==NULL) {
 			if (postprocessor != NULL) li.N_violations = 1;
@@ -1114,15 +1114,14 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	size_t i;
 	Linkage_info li;
 	PP_node * pp;
-	Postprocessor * postprocessor;
 	Sublinkage *sublinkage;
+	Postprocessor * postprocessor = sent->dict->postprocessor;
 	Parse_info pi = sent->parse_info;
 #ifdef USE_FAT_LINKAGES
 	analyze_context_t *actx = sent->analyze_ctxt;
 #endif /* USE_FAT_LINKAGES */
 
 	sublinkage = x_create_sublinkage(pi);
-	postprocessor = sent->dict->postprocessor;
 
 	compute_link_names(sent);
 	for (i=0; i<pi->N_links; i++)
@@ -1146,7 +1145,7 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	 * For this to work, however, you have to call "build_digraph"
 	 * first (as in analyze_fat_linkage), and then "free_digraph".
 	 */
-	pp = do_post_process(postprocessor, opts, sent, sublinkage, TRUE);
+	pp = do_post_process(postprocessor, opts, sent, sublinkage, true);
 
 	memset(&li, 0, sizeof(li));
 	li.N_violations = 0;
@@ -1281,7 +1280,8 @@ static inline void prt_dis_con_tree(Sentence sent, DIS_node *dis) {}
  */
 void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 {
-	int i, j, N_thin_links;
+	size_t i;
+	int j, N_thin_links;
 	DIS_node *d_root;
 	int num_sublinkages;
 	Sublinkage * sublinkage;
