@@ -49,9 +49,15 @@
 *
 ****************************************************************/
 
+/**
+ * For sorting the linkages in postprocessing
+ */
+
 static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
 {
-	/* for sorting the linkages in postprocessing */
+	/* Move the discarded entries to the end of the list */
+	if (p1->discarded || p2->discarded) return (p1->discarded - p2->discarded);
+
 	if (p1->N_violations != p2->N_violations) {
 		return (p1->N_violations - p2->N_violations);
 	}
@@ -79,6 +85,10 @@ static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
 static int CORP_compare_parse(Linkage_info * p1, Linkage_info * p2)
 {
 	double diff = p1->corpus_cost - p2->corpus_cost;
+
+	/* Move the discarded entries to the end of the list */
+	if (p1->discarded || p2->discarded) return (p1->discarded - p2->discarded);
+
 	if (fabs(diff) < 1.0e-5)
 		return VDAL_compare_parse(p1, p2);
 	if (diff < 0.0) return -1;
@@ -473,7 +483,279 @@ static void free_post_processing(Sentence sent)
 	}
 }
 
-static void post_process_linkages(Sentence sent, match_context_t* mchxt, 
+static void select_linkages(Sentence sent, match_context_t* mchxt,
+                            count_context_t* ctxt,
+                            Parse_Options opts)
+{
+	size_t in;
+	size_t N_linkages_found, N_linkages_alloced;
+	bool overflowed;
+	Linkage_info *link_info;
+
+	free_post_processing(sent);
+
+	overflowed = build_parse_set(sent, mchxt, ctxt, sent->null_count, opts);
+	print_time(opts, "Built parse set");
+
+	if (overflowed && (1 < opts->verbosity))
+	{
+		err_ctxt ec;
+		ec.sent = sent;
+		err_msg(&ec, Warn, "Warning: Count overflow.\n"
+		  "Considering a random subset of %zu of an unknown and large number of linkages\n",
+			opts->linkage_limit);
+	}
+	N_linkages_found = sent->num_linkages_found;
+
+	if (sent->num_linkages_found == 0)
+	{
+		sent->num_linkages_alloced = 0;
+		sent->num_linkages_post_processed = 0;
+		sent->num_valid_linkages = 0;
+#ifdef USE_FAT_LINKAGES
+		sent->num_thin_linkages = 0;
+#endif /* USE_FAT_LINKAGES */
+		sent->link_info = NULL;
+		return;
+	}
+
+	if (N_linkages_found > opts->linkage_limit)
+	{
+		N_linkages_alloced = opts->linkage_limit;
+		if (opts->verbosity > 1)
+		{
+			err_ctxt ec;
+			ec.sent = sent;
+			err_msg(&ec, Warn,
+			    "Warning: Considering a random subset of %zu of %zu linkages\n",
+			    N_linkages_alloced, N_linkages_found);
+		}
+	}
+	else
+	{
+		N_linkages_alloced = N_linkages_found;
+	}
+
+	link_info = linkage_info_new(N_linkages_alloced);
+
+	/* Generate an array of linkage indices to examine */
+	if (overflowed)
+	{
+		for (in=0; in < N_linkages_alloced; in++)
+		{
+			link_info[in].index = -(in+1);
+		}
+	}
+	else
+	{
+		if (opts->repeatable_rand)
+			sent->rand_state = N_linkages_found + sent->length;
+
+		for (in=0; in<N_linkages_alloced; in++)
+		{
+			size_t block_bottom, block_top;
+			double frac = (double) N_linkages_found;
+
+			frac /= (double) N_linkages_alloced;
+			block_bottom = (int) (((double) in) * frac);
+			block_top = (int) (((double) (in+1)) * frac);
+			link_info[in].index = block_bottom +
+				(rand_r(&sent->rand_state) % (block_top-block_bottom));
+		}
+	}
+
+	sent->link_info = link_info;
+	sent->num_linkages_alloced = N_linkages_alloced;
+	/* Later we subtract the number of invalid linkages */
+	sent->num_valid_linkages = N_linkages_alloced;
+
+	/* Allow use of both old and new pp order, for tests.
+	 * FIXME In release code this assignment can be removed if sane_morphism()
+	 * is modified to loop over num_linkages_alloced. */
+	sent->num_linkages_post_processed = N_linkages_alloced;
+}
+
+static void post_process_linkages(Sentence sent, Parse_Options opts)
+{
+	Linkage_info *lifo;
+	size_t in;
+	size_t N_linkages_post_processed = 0;
+	size_t N_valid_linkages = sent->num_valid_linkages;
+	size_t N_linkages_alloced = sent->num_linkages_alloced;
+#ifdef USE_FAT_LINKAGES
+	bool overflowed = (sent->link_info[0].index < 0);
+	size_t N_linkages_found = sent->num_linkages_found;
+	size_t N_thin_linkages;
+	bool only_canonical_allowed;
+	bool canonical;
+#endif /* USE_FAT_LINKAGES */
+
+#ifdef USE_FAT_LINKAGES
+	/* When we're processing only a small subset of the linkages,
+	 * don't worry about restricting the set we consider to be
+	 * canonical ones.  In the extreme case where we are only
+	 * generating 1 in a million linkages, it's very unlikely
+	 * that we'll hit two symmetric variants of the same linkage
+	 * anyway.
+	 */
+	only_canonical_allowed = !(overflowed || (N_linkages_found > 2*opts->linkage_limit));
+#endif /* USE_FAT_LINKAGES */
+
+	/* (optional) first pass: just visit the linkages */
+	/* The purpose of these two passes is to make the post-processing
+	 * more efficient.  Because (hopefully) by the time you do the
+	 * real work in the 2nd pass you've pruned the relevant rule set
+	 * in the first pass.
+	 */
+	if (sent->length >= opts->twopass_length)
+	{
+		for (in=0; in < N_linkages_alloced; in++)
+		{
+			lifo = &sent->link_info[in];
+			if (lifo->discarded || lifo->N_violations) continue;
+			extract_links(lifo->index, sent->parse_info);
+#ifdef USE_FAT_LINKAGES
+			if (set_has_fat_down(sent))
+			{
+				if (only_canonical_allowed && !is_canonical_linkage(sent)) continue;
+				analyze_fat_linkage(sent, opts, PP_FIRST_PASS);
+			}
+			else
+#endif /* USE_FAT_LINKAGES */
+			{
+				analyze_thin_linkage(sent, opts, PP_FIRST_PASS);
+			}
+			if ((9 == in%10) && resources_exhausted(opts->resources)) break;
+		}
+	}
+
+	/* second pass: actually perform post-processing */
+#ifdef USE_FAT_LINKAGES
+	N_thin_linkages = 0;
+#endif /* USE_FAT_LINKAGES */
+	for (in=0; in < N_linkages_alloced; in++)
+	{
+		lifo = &sent->link_info[in];
+		if (lifo->discarded) continue;
+		if (lifo->N_violations)
+		{
+			/* Arrange for displaying "Invalid morphism construction" sentences
+			 * if they are not discarded (for debug). */
+			N_linkages_post_processed++;
+			continue;
+		}
+		extract_links(lifo->index, sent->parse_info);
+#ifdef USE_FAT_LINKAGES
+		lifo->fat = false;
+		lifo->canonical = true;
+		if (set_has_fat_down(sent))
+		{
+			int index = lifo->index;
+			canonical = is_canonical_linkage(sent);
+			if (only_canonical_allowed && !canonical)
+			{
+				lifo->discarded = true;
+				continue;
+			}
+			*lifo = analyze_fat_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->index = index;
+			lifo->fat = true;
+			lifo->canonical = canonical;
+		}
+		else
+#endif /* USE_FAT_LINKAGES */
+		{
+			int index = lifo->index;
+			*lifo = analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->index = index;
+		}
+		if (0 == lifo->N_violations)
+		{
+#ifdef USE_FAT_LINKAGES
+			if (false == lifo->fat)
+				N_thin_linkages++;
+#endif /* USE_FAT_LINKAGES */
+		}
+		else
+		{
+			N_valid_linkages--;
+		}
+		lg_corpus_score(sent, lifo);
+		N_linkages_post_processed++;
+		if ((9 == in%10) && resources_exhausted(opts->resources)) break;
+	}
+
+	print_time(opts, "Postprocessed all linkages");
+
+#ifdef USE_FAT_LINKAGES
+	if (!resources_exhausted(opts->resources))
+	{
+		if ((N_linkages_post_processed == 0) &&
+		    (N_linkages_found > 0) &&
+		    (N_linkages_found <= opts->linkage_limit))
+		{
+			/* With the current parser, the following sentence will elicit
+			 * this error:
+			 *
+			 * Well, say, Joe, you can be Friar Tuck or Much the miller's
+			 * son, and lam me with a quarter-staff; or I'll be the Sheriff
+			 * of Nottingham and you be Robin Hood a little while and kill
+			 * me.
+			 */
+			err_ctxt ec;
+			ec.sent = sent;
+			err_msg(&ec, Error, "Error: None of the linkages is canonical\n"
+			          "\tN_linkages_post_processed=%zu "
+			          "N_linkages_found=%zu\n",
+			          N_linkages_post_processed,
+			          N_linkages_found);
+		}
+	}
+#endif /* USE_FAT_LINKAGES */
+
+	if (opts->verbosity > 1)
+	{
+		err_ctxt ec;
+		ec.sent = sent;
+		err_msg(&ec, Info, "Info: %zu of %zu linkages with no P.P. violations\n",
+				N_valid_linkages, N_linkages_post_processed);
+	}
+
+	sent->num_linkages_alloced = N_linkages_alloced;
+	sent->num_linkages_post_processed = N_linkages_post_processed;
+	sent->num_valid_linkages = N_valid_linkages;
+#ifdef USE_FAT_LINKAGES
+	sent->num_thin_linkages = N_thin_linkages;
+#endif /* USE_FAT_LINKAGES */
+}
+
+static void sort_linkages(Sentence sent, Parse_Options opts)
+{
+	qsort((void *)sent->link_info, sent->num_linkages_alloced,
+	      sizeof(Linkage_info),
+	      (int (*)(const void *, const void *))opts->cost_model.compare_fn);
+
+#if 0
+	/* num_linkages_post_processed sanity check (ONLY). */
+	{
+		size_t in;
+		size_t N_linkages_post_processed = 0;
+		for (in=0; in < sent->num_linkages_alloced; in++)
+		{
+			Linkage_info *lifo = &sent->link_info[in];
+			if (lifo->discarded) break;
+			N_linkages_post_processed++;
+		}
+		assert(sent->num_linkages_post_processed==N_linkages_post_processed,
+				 "Bad num_linkages_post_processed (%zu!=%zu)",
+				 sent->num_linkages_post_processed, N_linkages_post_processed);
+	}
+#endif
+
+	print_time(opts, "Sorted all linkages");
+}
+
+static void old_post_process_linkages(Sentence sent, match_context_t* mchxt,
                                   count_context_t* ctxt,
                                   Parse_Options opts)
 {
@@ -615,6 +897,7 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 			canonical = is_canonical_linkage(sent);
 			if (only_canonical_allowed && !canonical) continue;
 			*lifo = analyze_fat_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->index = indices[in];
 			lifo->fat = true;
 			lifo->canonical = canonical;
 		}
@@ -622,6 +905,7 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 #endif /* USE_FAT_LINKAGES */
 		{
 			*lifo = analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->index = indices[in];
 		}
 		if (0 == lifo->N_violations)
 		{
@@ -631,7 +915,6 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 				N_thin_linkages++;
 #endif /* USE_FAT_LINKAGES */
 		}
-		lifo->index = indices[in];
 		lg_corpus_score(sent, lifo);
 		N_linkages_post_processed++;
 		if ((9 == in%10) && resources_exhausted(opts->resources)) break;
@@ -641,8 +924,10 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 	qsort((void *)link_info, N_linkages_post_processed, sizeof(Linkage_info),
 		  (int (*)(const void *, const void *)) opts->cost_model.compare_fn);
 
+#ifdef USE_FAT_LINKAGES
 	if (!resources_exhausted(opts->resources))
 	{
+		/* This condition can ony happen w/ fat links */
 		if ((N_linkages_post_processed == 0) &&
 		    (N_linkages_found > 0) &&
 		    (N_linkages_found < opts->linkage_limit))
@@ -664,6 +949,7 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 			          N_linkages_found);
 		}
 	}
+#endif /* USE_FAT_LINKAGES */
 
 	if (opts->verbosity > 1)
 	{
@@ -684,7 +970,6 @@ static void post_process_linkages(Sentence sent, match_context_t* mchxt,
 	sent->link_info = link_info;
 
 	xfree(indices, N_linkages_alloced * sizeof(int));
-	/*if(N_valid_linkages == 0) free_andlists(sent); */
 }
 
 /***************************************************************
@@ -975,265 +1260,281 @@ static inline bool
 #define AFFIXTYPE_WORD		'w'	/* regular word */
 #define AFFIXTYPE_END		'b'	/* end of input word */
 
-static void sane_morphism(Sentence sent, Parse_Options opts)
+/** return true if its good, else return false */
+bool sane_linkage_morphism(Sentence sent, size_t lk, Parse_Options opts)
 {
-	size_t lk, i;
+	bool is_valid_morphism = false;
+	size_t i;
 	Parse_info pi = sent->parse_info;
 	Dictionary afdict = sent->dict->affix_table; /* for INFIX_MARK only */
-	char infix_mark = INFIX_MARK;
+	const char infix_mark = INFIX_MARK;
 	int * matched_alts = NULL;       /* number of morphemes that have matched
 												 * the chosen disjuncts for the
 												 * unsplit_word, so far (index: ai) */
 	size_t matched_alts_num = 0;     /* matched_alts number of elements */
+	char * const affix_types =
+	   alloca(sent->length*2 + 1);   /* affix types sequence */
+
 #define MATCHED_ALTS_MIN_INC 16     /* don't allocate matched_alts often */
 
-#if 0	/* Now it can always be enabled. Anyway, it is needed for "any". */
-	/* Skip checking, if dictionary specifies neither prefixes nor sufixes. */
-	if (NULL == afdict) return;
-	if ((0 == AFCLASS(afdict, AFDICT_PRE)->length) &&
-	    (0 == AFCLASS(afdict, AFDICT_MPRE)->length) &&
-	    (0 == AFCLASS(afdict, AFDICT_SUF)->length))
-	{
-		return;
-	}
-#endif
+	size_t numalt = 1;               /* number of alternatives */
+	size_t ai;                       /* index of checked alternative */
+	int unsplit_i = 0;               /* unsplit word index */
+	const char * unsplit = NULL;     /* unsplit word */
+	char * affix_types_p = affix_types;
+	/* If all the words are null - behave as if everything matched. */
+	bool match_found = true;        /* djw matched a morpheme */
+ 
+	Linkage_info * const lifo = &sent->link_info[lk];
 
-	for (lk = 0; lk < sent->num_linkages_alloced; lk++)
+	extract_links(lifo->index, pi);
+	*affix_types_p = '\0';
+	for (i=0; i<sent->length; i++)
 	{
-		Linkage_info *lifo = &sent->link_info[lk];
-		size_t numalt = 1;               /* number of alternatives */
-		size_t ai;                       /* index of checked alternative */
-		int unsplit_i = 0;               /* unsplit word index */
-		const char * unsplit = NULL;     /* unsplit word */
-		char affix_types[MAX_SENTENCE+1];/* affix types sequence */
-		char * affix_types_p = affix_types;
-		bool match_found = false;        /* djw matched a morpheme */
+		const char * djw;          /* disjunct word - the chosen word */
+		size_t djwlen;             /* disjunct total length */
+		size_t len;                /* disjunct length w/o subscript */
+		const char * mark;         /* char position of SUBSCRIPT_MARK */
+		bool empty_word = false;   /* is this an empty word? */
+		Disjunct * cdj = pi->chosen_disjuncts[i];
 
-		/* Don't bother with linkages that already failed post-processing... */
-		if (0 != lifo->N_violations)
-			continue;
-		extract_links(lifo->index, pi);
-		for (i=0; i<sent->length; i++)
+		lgdebug(+4, "Linkage %zu, word %zu/%zu\n", lk+1, i, sent->length);
+
+		/* Ignore island words */
+		if (NULL == cdj)
 		{
-			const char * djw;          /* disjunct word - the chosen word */
-			size_t djwlen;             /* disjunct total length */
-			size_t len;                /* disjunct length w/o subscript */
-			const char * mark;         /* char position of SUBSCRIPT_MARK */
-			bool empty_word = false;   /* is this an empty word? */
-			Disjunct * cdj = pi->chosen_disjuncts[i];
+			lgdebug(4, "%zu ignored island word\n", lk+1);
+			unsplit = NULL; /* mark it as island */
+			continue;
+		}
 
-			lgdebug(+4, "Linkage %zu, word %zu/%zu\n", lk+1, i, sent->length);
-
-			/* Ignore island words */
-			if (NULL == cdj)
+		if (NULL != sent->word[i].unsplit_word)
+		{
+			/* This is an input word - remember its parameters */
+			unsplit_i = i;
+			unsplit = sent->word[i].unsplit_word;
+			numalt = altlen(sent->word[i].alternatives);
+			if (numalt > matched_alts_num)
 			{
-				lgdebug(4, "%zu ignored island word\n", lk+1);
-				unsplit = NULL; /* mark it as island */
-				continue;
+				matched_alts_num = numalt + MATCHED_ALTS_MIN_INC;
+				matched_alts =
+					realloc(matched_alts, sizeof(*matched_alts)*matched_alts_num);
 			}
 
-			if (NULL != sent->word[i].unsplit_word)
-			{
-				/* This is an input word - remember its parameters */
-				unsplit_i = i;
-				unsplit = sent->word[i].unsplit_word;
-				numalt = altlen(sent->word[i].alternatives);
-				if (numalt > matched_alts_num)
-				{
-					matched_alts_num = numalt + MATCHED_ALTS_MIN_INC;
-					matched_alts =
-						realloc(matched_alts, sizeof(*matched_alts)*matched_alts_num);
-				}
-
-				lgdebug(4, "%zu unsplit word %s, alts:", lk+1, unsplit);
-				for (ai = 0; ai < numalt; ai++)
-				{
-					matched_alts[ai] = 0;
-					lgdebug(4, " %zu:%s", ai, sent->word[i].alternatives[ai]);
-				}
-				lgdebug(4, "\n");
-			}
-			if (NULL == unsplit)
-			{
-				lgdebug(4, "%zu ignore morphemes of an island word\n", lk+1);
-				continue;
-			}
-
-			djw = cdj->string;
-			djwlen = strlen(djw);
-			mark = strchr(djw, SUBSCRIPT_MARK);
-			len = NULL != mark ? (size_t)(mark - djw) : djwlen;
-
-			/* Find morpheme type */
-			if (is_AFFIXTYPE_EMPTY(infix_mark, djw, djwlen))
-			{
-				/* Ignore the empty word */;
-				empty_word = true;
-			}
-			else
-			if (is_AFFIXTYPE_SUFFIX(infix_mark, djw))
-			{
-				*affix_types_p = AFFIXTYPE_SUFFIX;
-			}
-			else
-			if (is_AFFIXTYPE_STEM(infix_mark, djw, len))
-			{
-				*affix_types_p = AFFIXTYPE_STEM;
-			}
-			else
-			if (is_AFFIXTYPE_PREFIX(infix_mark, djw, len))
-			{
-				*affix_types_p = AFFIXTYPE_PREFIX;
-			}
-			else
-			{
-				*affix_types_p = AFFIXTYPE_WORD;
-			}
-
-			lgdebug(4, "%zu djw=%s affixtype=%c\n",
-			        lk+1, djw, empty_word ? 'E' : *affix_types_p);
-
-			if (! empty_word) affix_types_p++;
-
-			lgdebug(4, "%zu djw %s matched alt#:", lk+1, djw);
-			match_found = false;
-
-			/* Compare the chosen word djw to the alternatives */
+			lgdebug(4, "%zu unsplit word %s, alts:", lk+1, unsplit);
 			for (ai = 0; ai < numalt; ai++)
 			{
-				const char * s, * t;
-				const char *a = sent->word[i].alternatives[ai];
-				char downcased[MAX_WORD+1] = "";
-
-				if (-1 == matched_alts[ai])
-					continue; /* already didn't match */
-
-				s = a;
-try_again:
-				t = djw;
-				//lgdebug(4, "\n%d COMPARING alt%d s %s djw %s: ", lk+1, (int)ai, s, t);
-				/* Rules of match:
-				 * A morpheme with a subscript needs an exact match to djw.
-				 * A morpheme w/o a subscript needs an exact match to djw
-				 * disregarding its subscript.
-				 * XXX To check: words that contain a dot as part of them. */
-				while ((*s != '\0' && *s != '[') && (*s == *t)) {s++; t++;}
-				/* Possibilities:
-				 *** Match (the last two are for words ending with [...]):
-				 * s==\0 && t==\0
-				 * s==\0 && t==SUBSCRIPT_MARK
-				 * s==\0 && t==[
-				 * s==[ && t==[
-				 *** Continue to check:
-				 * s==.	&& t==SUBSCRIPT_MARK
-				 */
-				if (*s == SUBSCRIPT_DOT && *t == SUBSCRIPT_MARK)
-				{
-					s++; t++;
-					while ((*s != '\0') && (*s == *t)) {s++; t++;}
-				}
-				if (((*s == '\0') &&
-				   ((*t == '\0') || (*t == SUBSCRIPT_MARK) || (*t == '[')))
-				   ||
-				   ((*s == '[') && (*t == '[')))
-				{
-					//lgdebug(4, "EQUAL\n");
-					lgdebug(4, " %zu", ai);
-					match_found = true;
-					/* Count matched morphemes in this alternative */
-					matched_alts[ai]++;
-				}
-				else {
-					//lgdebug(4,"NOTEQ\n");
-					/* If we are here, it didn't match. Is that because of
-					 * capitalization? Lets check. */
-					if ((sent->word[i].firstupper) &&
-						('\0' == downcased[0]) && ('\0' != a[0]))
-					{
-						downcase_utf8_str(downcased, a, MAX_WORD);
-						lgdebug(4, "\n");
-						lgdebug(4, "%zu downcasing %s>%s\n", lk+1, a, downcased);
-						s = downcased;
-						goto try_again;
-					}
-					/* No match, disregard this alternative from now on */
-					matched_alts[ai] = -1;
-				}
-			}
-			if (! match_found)
-			{
-				lgdebug(4, " none\n");
-				break;
+				matched_alts[ai] = 0;
+				lgdebug(4, " %zu:%s", ai, sent->word[i].alternatives[ai]);
 			}
 			lgdebug(4, "\n");
+		}
+		if (NULL == unsplit)
+		{
+			lgdebug(4, "%zu ignore morphemes of an island word\n", lk+1);
+			continue;
+		}
 
-			/* If this is the last morpheme of the alternatives,
-			 * then make sure all of them exist in the linkage */
-			if ((i+1 == sent->length) || (NULL != sent->word[i+1].unsplit_word))
+		djw = cdj->string;
+		djwlen = strlen(djw);
+		mark = strchr(djw, SUBSCRIPT_MARK);
+		len = NULL != mark ? (size_t)(mark - djw) : djwlen;
+
+		/* Find morpheme type */
+		if (is_AFFIXTYPE_EMPTY(infix_mark, djw, djwlen))
+		{
+			/* Ignore the empty word */;
+			empty_word = true;
+		}
+		else
+		if (is_AFFIXTYPE_SUFFIX(infix_mark, djw))
+		{
+			*affix_types_p = AFFIXTYPE_SUFFIX;
+		}
+		else
+		if (is_AFFIXTYPE_STEM(infix_mark, djw, len))
+		{
+			*affix_types_p = AFFIXTYPE_STEM;
+		}
+		else
+		if (is_AFFIXTYPE_PREFIX(infix_mark, djw, len))
+		{
+			*affix_types_p = AFFIXTYPE_PREFIX;
+		}
+		else
+		{
+			*affix_types_p = AFFIXTYPE_WORD;
+		}
+
+		lgdebug(4, "%zu djw=%s affixtype=%c\n",
+		        lk+1, djw, empty_word ? 'E' : *affix_types_p);
+
+		if (! empty_word) affix_types_p++;
+
+		lgdebug(4, "%zu djw %s matched alt#:", lk+1, djw);
+		match_found = false;
+
+		/* Compare the chosen word djw to the alternatives */
+		for (ai = 0; ai < numalt; ai++)
+		{
+			const char * s, * t;
+			const char *a = sent->word[i].alternatives[ai];
+			char downcased[MAX_WORD+1] = "";
+
+			if (-1 == matched_alts[ai])
+				continue; /* already didn't match */
+
+			s = a;
+try_again:
+			t = djw;
+			//lgdebug(4, "\n%d COMPARING alt%d s %s djw %s: ", lk+1, (int)ai, s, t);
+			/* Rules of match:
+			 * A morpheme with a subscript needs an exact match to djw.
+			 * A morpheme w/o a subscript needs an exact match to djw
+			 * disregarding its subscript.
+			 * XXX To check: words that contain a dot as part of them. */
+			while ((*s != '\0' && *s != '[') && (*s == *t)) {s++; t++;}
+			/* Possibilities:
+			 *** Match (the last two are for words ending with [...]):
+			 * s==\0 && t==\0
+			 * s==\0 && t==SUBSCRIPT_MARK
+			 * s==\0 && t==[
+			 * s==[ && t==[
+			 *** Continue to check:
+			 * s==.	&& t==SUBSCRIPT_MARK
+			 */
+			if (*s == SUBSCRIPT_DOT && *t == SUBSCRIPT_MARK)
 			{
-				int num_morphemes = i - unsplit_i + 1;
-
-				lgdebug(4, "%zu end of input word, num_morphemes %d\n",
-						lk+1, num_morphemes);
-				*affix_types_p++ = AFFIXTYPE_END;
-
-				/* Make sure that there exists an alternative in
-				 * which all the morphemes have been matched.
-				 * ??? This disallows island morphemes -
-				 * should we allow them? */
-				match_found = false;
-				for (ai = 0; ai < numalt; ai++)
+				s++; t++;
+				while ((*s != '\0') && (*s == *t)) {s++; t++;}
+			}
+			if (((*s == '\0') &&
+			   ((*t == '\0') || (*t == SUBSCRIPT_MARK) || (*t == '[')))
+			   ||
+			   ((*s == '[') && (*t == '[')))
+			{
+				//lgdebug(4, "EQUAL\n");
+				lgdebug(4, " %zu", ai);
+				match_found = true;
+				/* Count matched morphemes in this alternative */
+				matched_alts[ai]++;
+			}
+			else {
+				//lgdebug(4,"NOTEQ\n");
+				/* If we are here, it didn't match. Is that because of
+				 * capitalization? Lets check. */
+				if ((sent->word[i].firstupper) &&
+					('\0' == downcased[0]) && ('\0' != a[0]))
 				{
-					if (matched_alts[ai] == num_morphemes)
-					{
-						match_found = true;
-						break;
-					}
+					downcase_utf8_str(downcased, a, MAX_WORD);
+					lgdebug(4, "\n");
+					lgdebug(4, "%zu downcasing %s>%s\n", lk+1, a, downcased);
+					s = downcased;
+					goto try_again;
 				}
-
-				if (! match_found)
-				{
-					lgdebug(4, "%zu morphemes are missing in this linkage\n", lk+1);
-					break;
-				}
-				lgdebug(4, "%zu Perfect match\n", lk+1);
+				/* No match, disregard this alternative from now on */
+				matched_alts[ai] = -1;
 			}
 		}
-		*affix_types_p = '\0';
-
-		/* Check morpheme type combination.
-		 * If null_count > 0, the morpheme type combination may be invalid
-		 * due to morpheme islands, so skip this check. */
-		if (match_found && 0 == sent->null_count &&
-			(NULL != afdict) && (NULL != afdict->regex_root) &&
-			(NULL == match_regex(afdict->regex_root, affix_types)))
+		if (! match_found)
 		{
-			/* Morpheme type combination not valid */
+			lgdebug(4, " none\n");
+			break;
+		}
+		lgdebug(4, "\n");
+
+		/* If this is the last morpheme of the alternatives,
+		 * then make sure all of them exist in the linkage */
+		if ((i+1 == sent->length) || (NULL != sent->word[i+1].unsplit_word))
+		{
+			int num_morphemes = i - unsplit_i + 1;
+
+			lgdebug(4, "%zu end of input word, num_morphemes %d\n",
+					lk+1, num_morphemes);
+			*affix_types_p++ = AFFIXTYPE_END;
+
+			/* Make sure that there exists an alternative in
+			 * which all the morphemes have been matched.
+			 * ??? This disallows island morphemes -
+			 * should we allow them? */
 			match_found = false;
-			/* XXX we should have a better way to notify */
-			if (0 < verbosity)
-				printf("Warning: Invalid morpheme type combination %s, "
-				       "run with !bad and !verbosity=4 to debug\n", affix_types);
-		}
-		else
-			lgdebug(4, "%zu morpheme type combination %s\n", lk+1, affix_types);
+			for (ai = 0; ai < numalt; ai++)
+			{
+				if (matched_alts[ai] == num_morphemes)
+				{
+					match_found = true;
+					break;
+				}
+			}
 
-		if (match_found)
-		{
-			lgdebug(4, "%zu SUCCEEDED\n", lk+1);
-		}
-		else
-		{
-			/* Oh no ... invalid morpheme combination! */
-			sent->num_valid_linkages --;
-			lifo->N_violations ++;
-			lifo->pp_violation_msg = "Invalid morphism construction.";
-			lgdebug(4, "%zu FAILED, remaining %zu\n", lk+1,
-			        sent->num_valid_linkages);
+			if (! match_found)
+			{
+				lgdebug(4, "%zu morphemes are missing in this linkage\n", lk+1);
+				break;
+			}
+			lgdebug(4, "%zu Perfect match\n", lk+1);
 		}
 	}
+	*affix_types_p = '\0';
 
-	free(matched_alts);
+	/* Check morpheme type combination.
+	 * If null_count > 0, the morpheme type combination may be invalid
+	 * due to morpheme islands, so skip this check. */
+	if (match_found && (0 == sent->null_count) && ('\0' != affix_types[0]) &&
+		(NULL != afdict) && (NULL != afdict->regex_root) &&
+		(NULL == match_regex(afdict->regex_root, affix_types)))
+	{
+		/* Morpheme type combination not valid */
+		match_found = false;
+		/* XXX we should have a better way to notify */
+		if (0 < verbosity)
+			printf("Warning: Invalid morpheme type combination %s, "
+			       "run with !bad and !verbosity=4 to debug\n", affix_types);
+	}
+	else
+		lgdebug(4, "%zu morpheme type combination '%s'\n", lk+1, affix_types);
+
+	if (match_found)
+	{
+		lgdebug(4, "%zu SUCCEEDED\n", lk+1);
+	}
+	else
+	{
+		/* Oh no ... invalid morpheme combination! */
+		sent->num_valid_linkages --;
+		is_valid_morphism = false;
+		lifo->N_violations++;
+		lifo->pp_violation_msg = "Invalid morphism construction.";
+		if (!test_enabled("display-invalid-morphism")) lifo->discarded = true;
+		lgdebug(4, "%zu FAILED\n", lk+1);
+	}
+
+	if (matched_alts) free(matched_alts);
+
+	return is_valid_morphism;
+}
+
+static void sane_morphism(Sentence sent, Parse_Options opts)
+{
+	size_t N_invalid_morphism = 0;
+	size_t lk;
+
+	for (lk = 0; lk < sent->num_linkages_post_processed; lk++)
+	{
+		/* Don't bother with linkages that already failed post-processing... */
+		Linkage_info * const lifo = &sent->link_info[lk];
+		if (0 != lifo->N_violations) continue;
+
+      if (!sane_linkage_morphism(sent, lk, opts))
+          N_invalid_morphism ++;
+	}
+
+	if (opts->verbosity > 1)
+	{
+		prt_error("Info: sane_morphism(): %zu of %zu linkages had "
+					 "invalid morphology construction\n",
+		          N_invalid_morphism, sent->num_valid_linkages);
+	}
 }
 
 static void chart_parse(Sentence sent, Parse_Options opts)
@@ -1277,8 +1578,18 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		sent->num_linkages_found = (int) total;
 		print_time(opts, "Counted parses");
 
-		post_process_linkages(sent, mchxt, ctxt, opts);
-		sane_morphism(sent, opts);
+		if (test_enabled("old-pp-order"))
+		{
+			old_post_process_linkages(sent, mchxt, ctxt, opts);
+			sane_morphism(sent, opts);
+		}
+		else
+		{
+			select_linkages(sent, mchxt, ctxt, opts);
+			sane_morphism(sent, opts);
+			post_process_linkages(sent, opts);
+			sort_linkages(sent, opts);
+		}
 		if (sent->num_valid_linkages > 0) break;
 
 		/* If we are here, then no valid linakges were found.
