@@ -127,14 +127,14 @@ bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 	int x, y;
 	int dist;
 
-	if (a->label != b->label) return FALSE;
+	if (a->label != b->label) return false;
 
 	s = a->string;
 	t = b->string;
 
 	while (isupper((int)*s) || isupper((int)*t))
 	{
-		if (*s != *t) return FALSE;
+		if (*s != *t) return false;
 		s++;
 		t++;
 	}
@@ -152,7 +152,7 @@ bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 	}
 	/*	printf("M: a=%4s b=%4s  ap=%d bp=%d  aw=%d  bw=%d  a->ll=%d b->ll=%d  dist=%d\n",
 		   s, t, x, y, aw, bw, a->length_limit, b->length_limit, dist); */
-	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
+	if (dist > a->length_limit || dist > b->length_limit) return false;
 
 	if ((x == THIN_priority) && (y == THIN_priority))
 	{
@@ -171,9 +171,9 @@ bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 				t++;
 			}
 			else
-				return FALSE;
+				return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else if ((x == UP_priority) && (y == DOWN_priority))
 	{
@@ -194,9 +194,9 @@ bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 			{
 				s++;
 				t++;
-			} else return FALSE;
+			} else return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else if ((y == UP_priority) && (x == DOWN_priority))
 	{
@@ -208,12 +208,12 @@ bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 				t++;
 			}
 			else
-				return FALSE;
+				return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else
-		return FALSE;
+		return false;
 }
 
 #else /* not USE_FAT_LINKAGES */
@@ -278,7 +278,7 @@ find_table_pointer(count_context_t *ctxt,
 	                       (ctxt->current_resources != NULL) && 
 	                       resources_exhausted(ctxt->current_resources)))
 	{
-		ctxt->exhausted = TRUE;
+		ctxt->exhausted = true;
 		return table_store(ctxt, lw, rw, le, re, cost, 0);
 	}
 	else return NULL;
@@ -329,7 +329,7 @@ static s64 do_count(match_context_t *mchxt,
 	s64 total;
 	int start_word, end_word, w;
 	s64 leftcount, rightcount, pseudototal;
-	Boolean Lmatch, Rmatch;
+	bool Lmatch, Rmatch;
 	unsigned int lcost, rcost;
 
 	Match_node * m, *m1;
@@ -696,7 +696,7 @@ static void mark_region(match_context_t *mchxt, count_context_t *ctxt,
 		w = lw+1;
 		for (d = ctxt->local_sent[w].d; d != NULL; d = d->next) {
 			if ((d->left == NULL) && region_valid(mchxt, ctxt, w, rw, d->right, NULL)) {
-				d->marked = TRUE;
+				d->marked = true;
 				mark_region(mchxt, ctxt, w, rw, d->right, NULL);
 			}
 		}
@@ -739,7 +739,7 @@ static void mark_region(match_context_t *mchxt, count_context_t *ctxt,
 
 			if (left_valid && region_valid(mchxt, ctxt, w, rw, d->right, re))
 			{
-				d->marked = TRUE;
+				d->marked = true;
 				mark_region(mchxt, ctxt, w, rw, d->right, re);
 				mark_region(mchxt, ctxt, lw, w, le->next, d->left->next);
 				if (le->multi) mark_region(mchxt, ctxt, lw, w, le, d->left->next);
@@ -749,7 +749,7 @@ static void mark_region(match_context_t *mchxt, count_context_t *ctxt,
 
 			if (right_valid && region_valid(mchxt, ctxt, lw, w, le, d->left))
 			{
-				d->marked = TRUE;
+				d->marked = true;
 				mark_region(mchxt, ctxt, lw, w, le, d->left);
 				mark_region(mchxt, ctxt, w, rw, d->right->next,re->next);
 				if (d->right->multi) mark_region(mchxt, ctxt, w, rw, d->right, re->next);
@@ -759,7 +759,7 @@ static void mark_region(match_context_t *mchxt, count_context_t *ctxt,
 
 			if (left_valid && right_valid)
 			{
-				d->marked = TRUE;
+				d->marked = true;
 				mark_region(mchxt, ctxt, lw, w, le->next, d->left->next);
 				if (le->multi) mark_region(mchxt, ctxt, lw, w, le, d->left->next);
 				if (d->left->multi) mark_region(mchxt, ctxt, lw, w, le->next, d->left);
@@ -823,7 +823,7 @@ void conjunction_prune(Sentence sent, count_context_t *ctxt, Parse_Options opts)
 	   */
 	for (w=0; w<sent->length; w++) {
 		for (d=sent->word[w].d; d != NULL; d=d->next) {
-			d->marked = FALSE;
+			d->marked = false;
 		}
 	}
 
@@ -834,7 +834,7 @@ void conjunction_prune(Sentence sent, count_context_t *ctxt, Parse_Options opts)
 	for (d = sent->word[0].d; d != NULL; d = d->next) {
 		if ((d->left == NULL) && region_valid(mchxt, ctxt, 0, sent->length, d->right, NULL)) {
 			mark_region(mchxt, ctxt, 0, sent->length, d->right, NULL);
-			d->marked = TRUE;
+			d->marked = true;
 		}
 	}
 	mark_region(mchxt, ctxt, 0, sent->length, NULL, NULL);
@@ -850,7 +850,7 @@ void conjunction_prune(Sentence sent, count_context_t *ctxt, Parse_Options opts)
 				for (d = sent->word[w].d; d != NULL; d = d->next) {
 					if ((d->left == NULL) && region_valid(mchxt, ctxt, w, sent->length, d->right, NULL)) {
 						mark_region(mchxt, ctxt, w, sent->length, d->right, NULL);
-						d->marked = TRUE;
+						d->marked = true;
 					}
 				}
 			}
