@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -13,6 +12,8 @@
 
 /* verbosity global is held in utilities.c */
 extern int verbosity;          /* the verbosity level for error messages */
+extern char * debug;           /* comma-separated function list to debug */
+extern char * test;            /* comma-separated function list to debug */
 
 /* size of random table for computing the
    hash functions.  must be a power of 2 */
