@@ -11,6 +11,8 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <limits.h>  /* For UINT_MAX */
+
 #include "count.h"
 #include "fast-match.h"
 #include "extract-links.h"
@@ -125,8 +127,8 @@ Parse_info parse_info_new(int nwords)
 	pi->image_array = (Image_node **) xalloc(nwords * sizeof(Image_node *));
 	memset(pi->image_array, 0, nwords * sizeof(Image_node *));
 
-	pi->has_fat_down = (char *) xalloc(nwords * sizeof(Boolean));
-	memset(pi->has_fat_down, 0, nwords * sizeof(Boolean));
+	pi->has_fat_down = (char *) xalloc(nwords * sizeof(bool));
+	memset(pi->has_fat_down, 0, nwords * sizeof(bool));
 #endif /* USE_FAT_LINKAGES */
 
 	/* Alloc the x_table */
@@ -164,7 +166,7 @@ void free_parse_info(Parse_info pi)
 	xfree(pi->chosen_disjuncts, len * sizeof(Disjunct *));
 #ifdef USE_FAT_LINKAGES
 	xfree(pi->image_array, len * sizeof(Image_node*));
-	xfree(pi->has_fat_down, len * sizeof(Boolean));
+	xfree(pi->has_fat_down, len * sizeof(bool));
 #endif /* USE_FAT_LINKAGES */
 
 	for (i=0; i<pi->x_table_size; i++)
@@ -189,41 +191,34 @@ void free_parse_info(Parse_info pi)
 /**
  * Returns the pointer to this info, NULL if not there.
  */
-static X_table_connector * x_table_pointer(int lw, int rw, Connector *le, Connector *re,
-									unsigned int cost, Parse_info pi)
+static X_table_connector * x_table_pointer(int lw, int rw,
+                              Connector *le, Connector *re,
+                              unsigned int null_count, Parse_info pi)
 {
 	X_table_connector *t;
-	t = pi->x_table[pair_hash(pi->log2_x_table_size, lw, rw, le, re, cost)];
+	t = pi->x_table[pair_hash(pi->log2_x_table_size, lw, rw, le, re, null_count)];
 	for (; t != NULL; t = t->next) {
-		if ((t->lw == lw) && (t->rw == rw) && (t->le == le) && (t->re == re) && (t->cost == cost))  return t;
+		if ((t->lw == lw) && (t->rw == rw) &&
+		    (t->le == le) && (t->re == re) &&
+		    (t->null_count == null_count))  return t;
 	}
 	return NULL;
 }
 
-#if DEAD_CODE
-Parse_set * x_table_lookup(int lw, int rw, Connector *le, Connector *re,
-                           unsigned int cost, Parse_info pi)
-{
-	/* returns the count for this quintuple if there, -1 otherwise */
-	X_table_connector *t = x_table_pointer(lw, rw, le, re, cost, pi);
-
-	if (t == NULL) return -1; else return t->set;
-}
-#endif
-
 /**
  * Stores the value in the x_table.  Assumes it's not already there.
  */
-static X_table_connector * x_table_store(int lw, int rw, Connector *le, Connector *re,
-                                  unsigned int cost, Parse_info pi)
+static X_table_connector * x_table_store(int lw, int rw,
+                                  Connector *le, Connector *re,
+                                  unsigned int null_count, Parse_info pi)
 {
 	X_table_connector *t, *n;
 	unsigned int h;
 
 	n = (X_table_connector *) xalloc(sizeof(X_table_connector));
 	n->set = empty_set();
-	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->cost = cost;
-	h = pair_hash(pi->log2_x_table_size, lw, rw, le, re, cost);
+	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->null_count = null_count;
+	h = pair_hash(pi->log2_x_table_size, lw, rw, le, re, null_count);
 	t = pi->x_table[h];
 	n->next = t;
 	pi->x_table[h] = n;
@@ -232,9 +227,10 @@ static X_table_connector * x_table_store(int lw, int rw, Connector *le, Connecto
 
 #ifdef UNUSED_FUNCTION
 static void x_table_update(int lw, int rw, Connector *le, Connector *re,
-					unsigned int cost, Parse_set * set, Parse_info pi) {
+                unsigned int null_count, Parse_set * set, Parse_info pi)
+{
 	/* Stores the value in the x_table.  Unlike x_table_store, it assumes it's already there */
-	X_table_connector *t = x_table_pointer(lw, rw, le, re, cost, pi);
+	X_table_connector *t = x_table_pointer(lw, rw, le, re, null_count, pi);
 
 	assert(t != NULL, "This entry is supposed to be in the x_table.");
 	t->set = set;
@@ -253,13 +249,13 @@ static
 Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
                  count_context_t * ctxt,
                  Disjunct *ld, Disjunct *rd, int lw, int rw,
-                 Connector *le, Connector *re, unsigned int cost,
-                 Boolean islands_ok, Parse_info pi)
+                 Connector *le, Connector *re, unsigned int null_count,
+                 bool islands_ok, Parse_info pi)
 {
 	Disjunct * d, * dis;
 	int start_word, end_word, w;
-	Boolean Lmatch, Rmatch;
-	unsigned int lcost, rcost;
+	bool Lmatch, Rmatch;
+	unsigned int lnull_count, rnull_count;
 	int i, j;
 	Parse_set *ls[4], *rs[4], *lset, *rset;
 	Parse_choice * a_choice;
@@ -268,9 +264,9 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 	X_table_connector *xt;
 	s64 count;
 
-	assert(cost < 0x7fff, "mk_parse_set() called with cost < 0.");
+	assert(null_count < 0x7fff, "mk_parse_set() called with null_count < 0.");
 
-	count = table_lookup(ctxt, lw, rw, le, re, cost);
+	count = table_lookup(ctxt, lw, rw, le, re, null_count);
 
 	/*
 	  assert(count >= 0, "mk_parse_set() called on params that were not in the table.");
@@ -281,13 +277,13 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 
 	if ((count == 0) || (count == -1)) return NULL;
 
-	xt = x_table_pointer(lw, rw, le, re, cost, pi);
+	xt = x_table_pointer(lw, rw, le, re, null_count, pi);
 
 	if (xt != NULL) return xt->set;  /* we've already computed it */
 
 	/* Start it out with the empty set of options. */
 	/* This entry must be updated before we return. */
-	xt = x_table_store(lw, rw, le, re, cost, pi);
+	xt = x_table_store(lw, rw, le, re, null_count, pi);
 
 	xt->set->count = count;  /* the count we already computed */
 	/* this count is non-zero */
@@ -298,7 +294,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 	{
 		if (!islands_ok && (lw != -1)) return xt->set;
 
-		if (cost == 0) return xt->set;
+		if (null_count == 0) return xt->set;
 
 		w = lw + 1;
 		for (dis = sent->word[w].d; dis != NULL; dis = dis->next)
@@ -306,7 +302,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 			if (dis->left == NULL)
 			{
 				rs[0] = mk_parse_set(sent, mchxt, ctxt, dis, NULL, w, rw, dis->right,
-				                  NULL, cost-1, islands_ok, pi);
+				                  NULL, null_count-1, islands_ok, pi);
 				if (rs[0] == NULL) continue;
 				a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
 									   rs[0], w, rw, NULL, NULL,
@@ -315,7 +311,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 			}
 		}
 		rs[0] = mk_parse_set(sent, mchxt, ctxt, NULL, NULL, w, rw, NULL, NULL,
-		                  cost-1, islands_ok, pi);
+		                  null_count-1, islands_ok, pi);
 		if (rs[0] != NULL)
 		{
 			a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
@@ -344,16 +340,21 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 		end_word = re->word + 1;
 	}
 
+	/* This condition can never be true here. It is included so GCC will be able
+	 * to optimize the loop over "null_count".  Without this check, GCC thinks this
+	 * loop may be an infinite loop and it may omit some optimizations. */
+	if (UINT_MAX == null_count) return NULL;
+
 	for (w = start_word; w < end_word; w++)
 	{
 		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next)
 		{
 			d = m->d;
-			for (lcost = 0; lcost <= cost; lcost++)
+			for (lnull_count = 0; lnull_count <= null_count; lnull_count++)
 			{
-				rcost = cost-lcost;
-				/* now lcost and rcost are the costs we're assigning to
+				rnull_count = null_count-lnull_count;
+				/* now lnull_count and rnull_count are the null_counts we're assigning to
 				 * those parts respectively */
 
 				/* Now, we determine if (based on table only) we can see that
@@ -364,17 +365,17 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 				for (i=0; i<4; i++) {ls[i] = rs[i] = NULL;}
 				if (Lmatch)
 				{
-					ls[0] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left->next, lcost, islands_ok, pi);
-					if (le->multi) ls[1] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left->next, lcost, islands_ok, pi);
-					if (d->left->multi) ls[2] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left, lcost, islands_ok, pi);
-					if (le->multi && d->left->multi) ls[3] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
+					ls[0] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left->next, lnull_count, islands_ok, pi);
+					if (le->multi) ls[1] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left->next, lnull_count, islands_ok, pi);
+					if (d->left->multi) ls[2] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left, lnull_count, islands_ok, pi);
+					if (le->multi && d->left->multi) ls[3] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lnull_count, islands_ok, pi);
 				}
 				if (Rmatch)
 				{
-					rs[0] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re->next, rcost, islands_ok, pi);
-					if (d->right->multi) rs[1] = mk_parse_set(sent, mchxt, ctxt, d, rd, w,rw,d->right,re->next, rcost, islands_ok, pi);
-					if (re->multi) rs[2] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re, rcost, islands_ok, pi);
-					if (d->right->multi && re->multi) rs[3] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
+					rs[0] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re->next, rnull_count, islands_ok, pi);
+					if (d->right->multi) rs[1] = mk_parse_set(sent, mchxt, ctxt, d, rd, w,rw,d->right,re->next, rnull_count, islands_ok, pi);
+					if (re->multi) rs[2] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re, rnull_count, islands_ok, pi);
+					if (d->right->multi && re->multi) rs[3] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rnull_count, islands_ok, pi);
 				}
 
 				for (i=0; i<4; i++)
@@ -395,7 +396,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 				if (ls[0] != NULL || ls[1] != NULL || ls[2] != NULL || ls[3] != NULL)
 				{
 					/* evaluate using the left match, but not the right */
-					rset = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
+					rset = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rnull_count, islands_ok, pi);
 					if (rset != NULL)
 					{
 						for (i=0; i<4; i++)
@@ -415,7 +416,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 				     rs[1] != NULL || rs[2] != NULL || rs[3] != NULL))
 				{
 					/* evaluate using the right match, but not the left */
-					lset = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
+					lset = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lnull_count, islands_ok, pi);
 
 					if (lset != NULL)
 					{
@@ -441,7 +442,7 @@ Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
 }
 
 /**
- * return TRUE if and only if overflow in the number of parses occured.
+ * return TRUE if and only if overflow in the number of parses occurred.
  * Use a 64-bit int for counting.
  */
 static bool verify_set_node(Parse_set *set)
@@ -492,13 +493,13 @@ static bool verify_set(Parse_info pi)
 
 bool build_parse_set(Sentence sent, match_context_t *mchxt,
                     count_context_t *ctxt,
-                    unsigned int cost, Parse_Options opts)
+                    unsigned int null_count, Parse_Options opts)
 {
 	Parse_set * whole_set;
 
 	whole_set =
 		mk_parse_set(sent, mchxt, ctxt, 
-                NULL, NULL, -1, sent->length, NULL, NULL, cost+1,
+                NULL, NULL, -1, sent->length, NULL, NULL, null_count+1,
 		          opts->islands_ok, sent->parse_info);
 
 	if ((whole_set != NULL) && (whole_set->current != NULL)) {
