@@ -13,7 +13,8 @@
 #include "link-includes.h"
 #include "structures.h"
 
-void init_fast_matcher(Sentence sent);
-void free_fast_matcher(Sentence sent);
-void put_match_list(Sentence, Match_node *);
-Match_node * form_match_list(Sentence, int, Connector *, int, Connector *, int);
+match_context_t* alloc_fast_matcher(const Sentence);
+void free_fast_matcher(match_context_t*);
+
+void put_match_list(match_context_t *, Match_node *);
+Match_node * form_match_list(match_context_t *, int, Connector *, int, Connector *, int);
