@@ -44,7 +44,7 @@ bool post_process_match(const char *s, const char *t)
 	if (islower((int)*t)) t++; /* Skip head-dependent indicator */
 	while (isupper((int)*s) || isupper((int)*t))
 	{
-		if (*s != *t) return FALSE;
+		if (*s != *t) return false;
 		s++;
 		t++;
 	}
@@ -65,12 +65,12 @@ bool post_process_match(const char *s, const char *t)
 
 static int string_in_list(const char * s, const char * a[])
 {
-	/* returns FALSE if the string s does not match anything in
+	/* returns false if the string s does not match anything in
 		 the array.	The array elements are post-processing symbols */
 	int i;
 	for (i=0; a[i] != NULL; i++)
-		if (post_process_match(a[i], s)) return TRUE;
-	return FALSE;
+		if (post_process_match(a[i], s)) return true;
+	return false;
 }
 
 /**
@@ -89,7 +89,7 @@ static size_t find_domain_name(Postprocessor *pp, const char *link)
 	}
 }
 
-/** Returns TRUE if domain d1 is contained in domain d2 */
+/** Returns true if domain d1 is contained in domain d2 */
 static int contained_in(const Domain * d1, const Domain * d2,
                         const Sublinkage *sublinkage)
 {
@@ -97,10 +97,10 @@ static int contained_in(const Domain * d1, const Domain * d2,
 	List_o_links * lol;
 	memset(mark, 0, sublinkage->num_links*(sizeof(bool)));
 	for (lol=d2->lol; lol != NULL; lol = lol->next)
-		mark[lol->link] = TRUE;
+		mark[lol->link] = true;
 	for (lol=d1->lol; lol != NULL; lol = lol->next)
-		if (!mark[lol->link]) return FALSE;
-	return TRUE;
+		if (!mark[lol->link]) return false;
+	return true;
 }
 
 /** Returns the predicate "the given link is in the given domain" */
@@ -108,8 +108,8 @@ static bool link_in_domain(size_t link, const Domain * d)
 {
 	List_o_links * lol;
 	for (lol = d->lol; lol != NULL; lol = lol->next)
-		if (lol->link == link) return TRUE;
-	return FALSE;
+		if (lol->link == link) return true;
+	return false;
 }
 
 /* #define CHECK_DOMAIN_NESTING */
@@ -117,7 +117,7 @@ static bool link_in_domain(size_t link, const Domain * d)
 #if defined(CHECK_DOMAIN_NESTING)
 /* Although this is no longer used, I'm leaving the code here for future reference --DS 3/98 */
 
-/* Returns TRUE if the domains actually form a properly nested structure */
+/* Returns true if the domains actually form a properly nested structure */
 static bool check_domain_nesting(Postprocessor *pp, int num_links)
 {
 	Domain * d1, * d2;
@@ -138,10 +138,10 @@ static bool check_domain_nesting(Postprocessor *pp, int num_links)
 			for (i=0; i<num_links; i++)
 		counts[(int)mark[i]]++;/* (int) cast avoids compiler warning DS 7/97 */
 			if ((counts[1] > 0) && (counts[2] > 0) && (counts[3] > 0))
-		return FALSE;
+		return false;
 		}
 	}
-	return TRUE;
+	return true;
 }
 #endif
 
@@ -198,7 +198,7 @@ static void connectivity_dfs(Postprocessor *pp, Sublinkage *sublinkage,
                              int w, pp_linkset *ls)
 {
 	List_o_links *lol;
-	pp->visited[w] = TRUE;
+	pp->visited[w] = true;
 	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
 	{
 		if (!pp->visited[lol->word] &&
@@ -208,37 +208,13 @@ static void connectivity_dfs(Postprocessor *pp, Sublinkage *sublinkage,
 }
 #endif /* THIS_FUNCTION_IS_NOT_CURRENTLY_USED */
 
-static void mark_reachable_words(Postprocessor *pp, int w)
-{
-	List_o_links *lol;
-	if (pp->visited[w]) return;
-	pp->visited[w] = TRUE;
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
-		mark_reachable_words(pp, lol->word);
-}
-
-/**
- * Returns true if the linkage is connected, considering words
- * that have at least one edge....this allows conjunctive sentences
- * not to be thrown out. */
-static bool is_connected(Postprocessor *pp)
-{
-	size_t i;
-	for (i=0; i<pp->pp_data.length; i++)
-		pp->visited[i] = (pp->pp_data.word_links[i] == NULL);
-	mark_reachable_words(pp, 0);
-	for (i=0; i<pp->pp_data.length; i++)
-		if (!pp->visited[i]) return FALSE;
-	return TRUE;
-}
-
 static void chk_d_type(PP_node* ppn, size_t idx)
 {
 	if (ppn->dtsz <= idx)
 	{
 		size_t oldsz = ppn->dtsz;
 		ppn->dtsz = 2 * ppn->dtsz + 10;
-		ppn->d_type_array = xrealloc(ppn->d_type_array, 
+		ppn->d_type_array = xrealloc(ppn->d_type_array,
 			oldsz * sizeof(D_type_list*), ppn->dtsz * sizeof(D_type_list*));
 	}
 }
@@ -248,9 +224,9 @@ static void build_type_array(Postprocessor *pp)
 	D_type_list * dtl;
 	size_t d;
 	List_o_links * lol;
-	for (d=0; d<pp->pp_data.N_domains; d++)
+	for (d = 0; d < pp->pp_data.N_domains; d++)
 	{
-		for (lol=pp->pp_data.domain_array[d].lol; lol != NULL; lol = lol->next)
+		for (lol = pp->pp_data.domain_array[d].lol; lol != NULL; lol = lol->next)
 		{
 			chk_d_type(pp->pp_node, lol->link);
 			dtl = (D_type_list *) xalloc(sizeof(D_type_list));
@@ -261,15 +237,20 @@ static void build_type_array(Postprocessor *pp)
 	}
 }
 
+#ifndef USE_FAT_LINKAGES
+static
+#endif /* USE_FAT_LINKAGES */
 void free_d_type(D_type_list * dtl)
 {
 	D_type_list * dtlx;
-	for (; dtl!=NULL; dtl=dtlx) {
+	for (; dtl != NULL; dtl = dtlx)
+	{
 		dtlx = dtl->next;
 		xfree((void*) dtl, sizeof(D_type_list));
 	}
 }
 
+#ifdef USE_FAT_LINKAGES
 D_type_list * copy_d_type(const D_type_list * dtl)
 {
 	D_type_list *dtlx, *dtlcurr=NULL, *dtlhead=NULL;
@@ -290,6 +271,7 @@ D_type_list * copy_d_type(const D_type_list * dtl)
 	}
 	return dtlhead;
 }
+#endif /* USE_FAT_LINKAGES */
 
 /** free the pp node from last time */
 static void free_pp_node(Postprocessor *pp)
@@ -333,43 +315,50 @@ static void clear_pp_node(Postprocessor *pp)
 /************************ rule application *******************************/
 
 static bool apply_rules(Postprocessor *pp,
-							 bool (applyfn) (Postprocessor *, Sublinkage *, pp_rule *),
-							 Sublinkage *sublinkage,
-							 pp_rule *rule_array,
-							 const char **msg)
+                        bool (applyfn) (Postprocessor *, Sublinkage *, pp_rule *),
+                        Sublinkage *sublinkage,
+                        pp_rule *rule_array,
+                        const char **msg)
 {
 	int i;
-	for (i=0; (*msg=rule_array[i].msg)!=NULL; i++)
-		if (!applyfn(pp, sublinkage, &(rule_array[i]))) return false;
+	for (i = 0; (*msg = rule_array[i].msg) != NULL; i++)
+	{
+		if (!applyfn(pp, sublinkage, &(rule_array[i])))
+		{
+			rule_array[i].use_count ++;
+			return false;
+		}
+	}
 	return true;
 }
 
 static bool
 apply_relevant_rules(Postprocessor *pp,
-						 bool(applyfn)(Postprocessor *, Sublinkage*, pp_rule *),
-						 Sublinkage *sublinkage,
-						 pp_rule *rule_array,
-						 int *relevant_rules,
-						 const char **msg)
+                     bool (applyfn)(Postprocessor *, Sublinkage*, pp_rule *),
+                     Sublinkage *sublinkage,
+                     pp_rule *rule_array,
+                     int *relevant_rules,
+                     const char **msg)
 {
 	int i, idx;
 
 	/* if we didn't accumulate link names for this sentence, we need to apply
 		 all rules */
-	if (pp_linkset_population(pp->set_of_links_of_sentence)==0) {
-			return apply_rules(pp, applyfn, sublinkage, rule_array, msg);
+	if (pp_linkset_population(pp->set_of_links_of_sentence) == 0) {
+		return apply_rules(pp, applyfn, sublinkage, rule_array, msg);
 	}
 
 	/* we did, and we don't */
-	for (i=0; (idx=relevant_rules[i])!=-1; i++) {
-			*msg = rule_array[idx].msg;
-			if (!applyfn(pp, sublinkage, &(rule_array[idx]))) return false;
+	for (i = 0; (idx = relevant_rules[i]) != -1; i++)
+	{
+		*msg = rule_array[idx].msg;
+		if (!applyfn(pp, sublinkage, &(rule_array[idx]))) return false;
 	}
 	return true;
 }
 
 /**
- * returns TRUE if and only if all groups containing the specified link
+ * returns true if and only if all groups containing the specified link
  * contain at least one from the required list.	(as determined by exact
  * string matching)
  */
@@ -379,69 +368,74 @@ apply_contains_one(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 	DTreeLeaf * dtl;
 	size_t d, count;
 	for (d=0; d<pp->pp_data.N_domains; d++)
-		{
-			for (dtl = pp->pp_data.domain_array[d].child;
-			 dtl != NULL &&
-				 !post_process_match(rule->selector,
-								 sublinkage->link[dtl->link]->link_name);
-			 dtl = dtl->next) {}
-			if (dtl != NULL)
+	{
+		for (dtl = pp->pp_data.domain_array[d].child;
+		     dtl != NULL &&
+		        !post_process_match(rule->selector,
+		           sublinkage->link[dtl->link]->link_name);
+		     dtl = dtl->next) {}
+		if (dtl != NULL)
 		{
 			/* selector link of rule appears in this domain */
 			count=0;
 			for (dtl = pp->pp_data.domain_array[d].child; dtl != NULL; dtl = dtl->next)
+			{
 				if (string_in_list(sublinkage->link[dtl->link]->link_name,
 									 rule->link_array))
-					{
-				count=1;
-				break;
-					}
-			if (count == 0) return FALSE;
+				{
+					count=1;
+					break;
+				}
+			}
+			if (count == 0) return false;
 		}
-		}
-	return TRUE;
+	}
+	return true;
 }
 
 
 /**
- * Returns TRUE if and only if:
+ * Returns true if and only if:
  * all groups containing the selector link do not contain anything
  * from the link_array contained in the rule. Uses exact string matching.
  */
 static bool
-apply_contains_none(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
+apply_contains_none(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
 	size_t d;
 	for (d=0; d<pp->pp_data.N_domains; d++)
 	{
 		DTreeLeaf * dtl;
 		for (dtl = pp->pp_data.domain_array[d].child;
-			 dtl != NULL &&
-				 !post_process_match(rule->selector,
-								 sublinkage->link[dtl->link]->link_name);
-			 dtl = dtl->next) {}
+		     dtl != NULL &&
+		         !post_process_match(rule->selector,
+		                  sublinkage->link[dtl->link]->link_name);
+		     dtl = dtl->next) {}
 		if (dtl != NULL)
 		{
 			/* selector link of rule appears in this domain */
 			for (dtl = pp->pp_data.domain_array[d].child; dtl != NULL; dtl = dtl->next)
+			{
 				if (string_in_list(sublinkage->link[dtl->link]->link_name,
-									 rule->link_array))
-					return FALSE;
+				                   rule->link_array))
+					return false;
+			}
 		}
 	}
-	return TRUE;
+	return true;
 }
 
 /**
- * Returns TRUE if and only if
+ * Returns true if and only if
  * (1) the sentence doesn't contain the selector link for the rule, or
  * (2) it does, and it also contains one or more from the rule's link set
  */
 static bool
-apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
+apply_contains_one_globally(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
-	size_t i,j,count;
-	for (i=0; i<sublinkage->num_links; i++) {
+	size_t i, j, count;
+	for (i = 0; i < sublinkage->num_links; i++)
+	{
 #ifdef USE_FAT_LINKAGES
 		if (sublinkage->link[i]->lw == SIZE_MAX) continue;
 #else
@@ -449,11 +443,12 @@ apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *ru
 #endif
 		if (post_process_match(rule->selector,sublinkage->link[i]->link_name)) break;
 	}
-	if (i == sublinkage->num_links) return TRUE;
+	if (i == sublinkage->num_links) return true;
 
 	/* selector link of rule appears in sentence */
 	count = 0;
-	for (j=0; j<sublinkage->num_links && count==0; j++) {
+	for (j = 0; j < sublinkage->num_links && count == 0; j++)
+	{
 #ifdef USE_FAT_LINKAGES
 		if (sublinkage->link[j]->lw == SIZE_MAX) continue;
 #else
@@ -461,20 +456,11 @@ apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *ru
 #endif
 		if (string_in_list(sublinkage->link[j]->link_name, rule->link_array))
 		{
-			count=1;
+			count = 1;
 			break;
 		}
 	}
-	if (count == 0) return FALSE; else return TRUE;
-}
-
-static bool
-apply_connected(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
-{
-	/* There is actually just one (or none, if user didn't specify it)
-		 rule asserting that linkage is connected. */
-	if (!is_connected(pp)) return false;
-	return true;
+	if (count == 0) return false; else return true;
 }
 
 /**
@@ -484,17 +470,17 @@ apply_connected(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
  * link are still connected.
  */
 static void reachable_without_dfs(Postprocessor *pp,
-                Sublinkage *sublinkage, size_t a, size_t b, size_t w)
+                    Sublinkage *sublinkage, size_t a, size_t b, size_t w)
 {
 	/* This is a depth first search of words reachable from w, excluding
 	 * any direct edge between word a and word b. */
 	List_o_links *lol;
-	pp->visited[w] = TRUE;
+	pp->visited[w] = true;
 	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
 	{
-		if (!pp->visited[lol->word] && 
-          !(w == a && lol->word == b) && 
-          !(w == b && lol->word == a))
+		if (!pp->visited[lol->word] &&
+		    !(w == a && lol->word == b) &&
+		    !(w == b && lol->word == a))
 		{
 				reachable_without_dfs(pp, sublinkage, a, b, lol->word);
 		}
@@ -502,18 +488,18 @@ static void reachable_without_dfs(Postprocessor *pp,
 }
 
 /**
- * Returns TRUE if the linkage is connected when ignoring the links
+ * Returns true if the linkage is connected when ignoring the links
  * whose names are in the given list of link names.
- * Actually, what it does is this: it returns FALSE if the connectivity
+ * Actually, what it does is this: it returns false if the connectivity
  * of the subgraph reachable from word 0 changes as a result of deleting
  * these links.
  */
 static bool
-apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage, pp_rule *rule)
+apply_must_form_a_cycle(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
 	List_o_links *lol;
 	size_t w;
-	for (w=0; w<pp->pp_data.length; w++)
+	for (w = 0; w < pp->pp_data.length; w++)
 	{
 		for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
 		{
@@ -521,7 +507,7 @@ apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage, pp_rule *rule)
 			if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->link_name)) continue;
 			memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
 			reachable_without_dfs(pp, sublinkage, w, lol->word, w);
-			if (!pp->visited[lol->word]) return FALSE;
+			if (!pp->visited[lol->word]) return false;
 		}
 	}
 
@@ -550,16 +536,16 @@ apply_bounded(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 	char d_type;
 	List_o_links * lol;
 	d_type = rule->domain;
-	for (d=0; d<pp->pp_data.N_domains; d++)
+	for (d = 0; d < pp->pp_data.N_domains; d++)
 	{
 		if (pp->pp_data.domain_array[d].type != d_type) continue;
 		lw = sublinkage->link[pp->pp_data.domain_array[d].start_link]->lw;
 		for (lol = pp->pp_data.domain_array[d].lol; lol != NULL; lol = lol->next)
 		{
-			if (sublinkage->link[lol->link]->lw < lw) return FALSE;
+			if (sublinkage->link[lol->link]->lw < lw) return false;
 		}
 	}
-	return TRUE;
+	return true;
 }
 
 /**
@@ -573,10 +559,10 @@ static void build_graph(Postprocessor *pp, Sublinkage *sublinkage)
 	size_t i, link;
 	List_o_links * lol;
 
-	for (i=0; i<pp->pp_data.length; i++)
+	for (i = 0; i < pp->pp_data.length; i++)
 		pp->pp_data.word_links[i] = NULL;
 
-	for (link=0; link<sublinkage->num_links; link++)
+	for (link = 0; link < sublinkage->num_links; link++)
 	{
 #ifdef USE_FAT_LINKAGES
 		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
@@ -608,9 +594,9 @@ static void build_graph(Postprocessor *pp, Sublinkage *sublinkage)
 }
 
 static void setup_domain_array(Postprocessor *pp,
-									 int n, const char *string, int start_link)
+                               int n, const char *string, int start_link)
 {
-	/* set pp->visited[i] to FALSE */
+	/* set pp->visited[i] to false */
 	memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
 	pp->pp_data.domain_array[n].string = string;
 	pp->pp_data.domain_array[n].lol    = NULL;
@@ -629,40 +615,50 @@ static void add_link_to_domain(Postprocessor *pp, int link)
 }
 
 static void depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-									 size_t w, size_t root, size_t start_link)
+                               size_t w, size_t root, size_t start_link)
 {
 	List_o_links *lol;
-	pp->visited[w] = TRUE;
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
-		if (lol->word < w && lol->link != start_link) {
+	pp->visited[w] = true;
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
+		if (lol->word < w && lol->link != start_link)
+		{
 			add_link_to_domain(pp, lol->link);
 		}
 	}
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
 		if (!pp->visited[lol->word] && (lol->word != root) &&
-		!(lol->word < root && lol->word < w &&
-					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->link_name)))
+		       !(lol->word < root && lol->word < w &&
+		       pp_linkset_match(pp->knowledge->restricted_links,
+		                sublinkage->link[lol->link]->link_name)))
+		{
 			depth_first_search(pp, sublinkage, lol->word, root, start_link);
+		}
 	}
 }
 
 static void bad_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-                                 size_t w, size_t root, size_t start_link)
+                                   size_t w, size_t root, size_t start_link)
 {
 	List_o_links * lol;
-	pp->visited[w] = TRUE;
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
-		if ((lol->word < w)	&& (lol->link != start_link) && (w != root)) {
+	pp->visited[w] = true;
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
+		if ((lol->word < w)	&& (lol->link != start_link) && (w != root))
+		{
 			add_link_to_domain(pp, lol->link);
 		}
 	}
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
 		if ((!pp->visited[lol->word]) && !(w == root && lol->word < w) &&
-		!(lol->word < root && lol->word < w &&
-					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->link_name)))
+		     !(lol->word < root && lol->word < w &&
+		          pp_linkset_match(pp->knowledge->restricted_links,
+		                sublinkage->link[lol->link]->link_name)))
+		{
 			bad_depth_first_search(pp, sublinkage, lol->word, root, start_link);
+		}
 	}
 }
 
@@ -670,40 +666,52 @@ static void d_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
                     size_t w, size_t root, size_t right, size_t start_link)
 {
 	List_o_links * lol;
-	pp->visited[w] = TRUE;
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
-		if ((lol->word < w) && (lol->link != start_link) && (w != root)) {
+	pp->visited[w] = true;
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
+		if ((lol->word < w) && (lol->link != start_link) && (w != root))
+		{
 			add_link_to_domain(pp, lol->link);
 		}
 	}
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
 		if (!pp->visited[lol->word] && !(w == root && lol->word >= right) &&
-		!(w == root && lol->word < root) &&
-		!(lol->word < root && lol->word < w &&
-					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->link_name)))
+		    !(w == root && lol->word < root) &&
+		       !(lol->word < root && lol->word < w &&
+		          pp_linkset_match(pp->knowledge->restricted_links,
+		                   sublinkage->link[lol->link]->link_name)))
+		{
 			d_depth_first_search(pp,sublinkage,lol->word,root,right,start_link);
+		}
 	}
 }
 
 static void left_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-                               size_t w, size_t right, size_t start_link)
+                                    size_t w, size_t right, size_t start_link)
 {
 	List_o_links *lol;
-	pp->visited[w] = TRUE;
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
-		if (lol->word < w && lol->link != start_link) {
+	pp->visited[w] = true;
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
+		if (lol->word < w && lol->link != start_link)
+		{
 			add_link_to_domain(pp, lol->link);
 		}
 	}
-	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next) {
+	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
+	{
 		if (!pp->visited[lol->word] && (lol->word != right))
+		{
 			depth_first_search(pp, sublinkage, lol->word, right, start_link);
+		}
 	}
 }
 
 static int domain_compare(const Domain * d1, const Domain * d2)
-{ return (d1->size-d2->size); /* for sorting the domains by size */ }
+{
+	return (d1->size - d2->size); /* for sorting the domains by size */
+}
 
 static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 {
@@ -711,7 +719,8 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 	const char *s;
 	pp->pp_data.N_domains = 0;
 
-	for (link = 0; link<sublinkage->num_links; link++) {
+	for (link = 0; link<sublinkage->num_links; link++)
+	{
 #ifdef USE_FAT_LINKAGES
 		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
 #else
@@ -738,7 +747,7 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 			/* always add the starter link to its urfl domain */
 			add_link_to_domain(pp, link);
 			bad_depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
-							 sublinkage->link[link]->lw, link);
+			                       sublinkage->link[link]->lw, link);
 			pp->pp_data.N_domains++;
 			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
@@ -748,8 +757,8 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 			/* do not add the starter link to its urfl_only domain */
 			d_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
-							 sublinkage->link[link]->lw,
-							 sublinkage->link[link]->rw, link);
+			                     sublinkage->link[link]->lw,
+			                     sublinkage->link[link]->rw, link);
 			pp->pp_data.N_domains++;
 			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
@@ -759,7 +768,7 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 			/* do not add the starter link to a left domain */
 			left_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
-							 sublinkage->link[link]->rw, link);
+			                        sublinkage->link[link]->rw, link);
 			pp->pp_data.N_domains++;
 			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
@@ -772,7 +781,8 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 		(int (*)(const void *, const void *)) domain_compare);
 
 	/* sanity check: all links in all domains have a legal domain name */
-	for (d=0; d<pp->pp_data.N_domains; d++) {
+	for (d = 0; d < pp->pp_data.N_domains; d++)
+	{
 		i = find_domain_name(pp, pp->pp_data.domain_array[d].string);
 		if (i == SIZE_MAX)
 			 prt_error("Error: post_process(): Need an entry for %s in LINK_TYPE_TABLE",
@@ -788,15 +798,18 @@ static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 	if (pp->pp_data.N_domains > 0)
 	{
 		pp->pp_data.domain_array[pp->pp_data.N_domains-1].parent = NULL;
-		for (d=0; d < pp->pp_data.N_domains-1; d++) {
-			for (d1 = d+1; d1 < pp->pp_data.N_domains; d1++) {
-				if (contained_in(&pp->pp_data.domain_array[d],&pp->pp_data.domain_array[d1],sublinkage))
+		for (d=0; d < pp->pp_data.N_domains-1; d++)
+		{
+			for (d1 = d+1; d1 < pp->pp_data.N_domains; d1++)
+			{
+				if (contained_in(&pp->pp_data.domain_array[d], &pp->pp_data.domain_array[d1], sublinkage))
 				{
 					pp->pp_data.domain_array[d].parent = &pp->pp_data.domain_array[d1];
 					break;
 				}
 			}
-			if (d1 == pp->pp_data.N_domains) {
+			if (d1 == pp->pp_data.N_domains)
+			{
 				/* we know this domain is a root of a new tree */
 				pp->pp_data.domain_array[d].parent = NULL;
 			}
@@ -804,17 +817,21 @@ static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 	}
 	/* the parent links of domain nodes have been established.
 		 now do the leaves */
-	for (d=0; d < pp->pp_data.N_domains; d++) {
+	for (d = 0; d < pp->pp_data.N_domains; d++)
+	{
 		pp->pp_data.domain_array[d].child = NULL;
 	}
-	for (link=0; link < sublinkage->num_links; link++) {
+	for (link=0; link < sublinkage->num_links; link++)
+	{
 #ifdef USE_FAT_LINKAGES
 		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
 #else
 		assert (sublinkage->link[link]->lw != SIZE_MAX);
 #endif
-		for (d=0; d<pp->pp_data.N_domains; d++) {
-			if (link_in_domain(link, &pp->pp_data.domain_array[d])) {
+		for (d=0; d<pp->pp_data.N_domains; d++)
+		{
+			if (link_in_domain(link, &pp->pp_data.domain_array[d]))
+			{
 				dtl = (DTreeLeaf *) xalloc(sizeof(DTreeLeaf));
 				dtl->link = link;
 				dtl->parent = &pp->pp_data.domain_array[d];
@@ -832,11 +849,11 @@ internal_process(Postprocessor *pp, Sublinkage *sublinkage, const char **msg)
 	size_t i;
 	/* quick test: try applying just the relevant global rules */
 	if (!apply_relevant_rules(pp,apply_contains_one_globally,
-								sublinkage,
-								pp->knowledge->contains_one_rules,
-								pp->relevant_contains_one_rules, msg))
+	                          sublinkage,
+	                          pp->knowledge->contains_one_rules,
+	                          pp->relevant_contains_one_rules, msg))
 	{
-		for (i=0; i<pp->pp_data.length; i++)
+		for (i = 0; i < pp->pp_data.length; i++)
 			pp->pp_data.word_links[i] = NULL;
 		pp->pp_data.N_domains = 0;
 		return -1;
@@ -857,16 +874,14 @@ internal_process(Postprocessor *pp, Sublinkage *sublinkage, const char **msg)
 	/* The order below should be optimal for most cases */
 	if (!apply_relevant_rules(pp, apply_contains_one, sublinkage,
 								pp->knowledge->contains_one_rules,
-								pp->relevant_contains_one_rules, msg))	return 1;
-	if (!apply_relevant_rules(pp,apply_contains_none, sublinkage,
+								pp->relevant_contains_one_rules, msg)) return 1;
+	if (!apply_relevant_rules(pp, apply_contains_none, sublinkage,
 								pp->knowledge->contains_none_rules,
 									pp->relevant_contains_none_rules, msg)) return 1;
-	if (!apply_rules(pp,apply_must_form_a_cycle, sublinkage,
-					 pp->knowledge->form_a_cycle_rules,msg))		 return 1;
-	if (!apply_rules(pp,apply_connected, sublinkage,
-					 pp->knowledge->connected_rules, msg))						return 1;
-	if (!apply_rules(pp,apply_bounded, sublinkage,
-					 pp->knowledge->bounded_rules, msg))							return 1;
+	if (!apply_rules(pp, apply_must_form_a_cycle, sublinkage,
+					 pp->knowledge->form_a_cycle_rules,msg)) return 1;
+	if (!apply_rules(pp, apply_bounded, sublinkage,
+					 pp->knowledge->bounded_rules, msg)) return 1;
 	return 0; /* This linkage satisfied all the rules */
 }
 
@@ -875,21 +890,21 @@ internal_process(Postprocessor *pp, Sublinkage *sublinkage, const char **msg)
  * Call this (a) after having called post_process_scan_linkage() on all
  * generated linkages, but (b) before calling post_process() on any
  * particular linkage. Here we mark all rules which we know (from having
- * accumulated a set of link names appearing in *any* linkage) won't
+ * accumulated a set of link names appearing in *any* linkage) that won't
  * ever be needed.
  */
 static void prune_irrelevant_rules(Postprocessor *pp)
 {
-	 pp_rule *rule;
-	 int coIDX, cnIDX, rcoIDX=0, rcnIDX=0;
+	pp_rule *rule;
+	int coIDX, cnIDX, rcoIDX = 0, rcnIDX = 0;
 
 	/* If we didn't scan any linkages, there's no pruning to be done. */
-	if (pp_linkset_population(pp->set_of_links_of_sentence)==0) return;
+	if (pp_linkset_population(pp->set_of_links_of_sentence) == 0) return;
 
-	for (coIDX=0;;coIDX++)
+	for (coIDX = 0; ; coIDX++)
 	{
 		rule = &(pp->knowledge->contains_one_rules[coIDX]);
-		if (rule->msg==NULL) break;
+		if (rule->msg == NULL) break;
 		if (pp_linkset_match_bw(pp->set_of_links_of_sentence, rule->selector))
 		{
 			/* mark rule as being relevant to this sentence */
@@ -899,10 +914,10 @@ static void prune_irrelevant_rules(Postprocessor *pp)
 	}
 	pp->relevant_contains_one_rules[rcoIDX] = -1;	/* end sentinel */
 
-	for (cnIDX=0;;cnIDX++)
+	for (cnIDX = 0; ; cnIDX++)
 	{
 		rule = &(pp->knowledge->contains_none_rules[cnIDX]);
-		if (rule->msg==NULL) break;
+		if (rule->msg == NULL) break;
 		if (pp_linkset_match_bw(pp->set_of_links_of_sentence, rule->selector))
 		{
 			pp->relevant_contains_none_rules[rcnIDX++] = cnIDX;
@@ -913,9 +928,9 @@ static void prune_irrelevant_rules(Postprocessor *pp)
 
 	if (verbosity > 1)
 	{
-		printf("Saw %i unique link names in all linkages.\n",
+		printf("PP: Saw %zd unique link names in all linkages.\n",
 				pp_linkset_population(pp->set_of_links_of_sentence));
-		printf("Using %i 'contains one' rules and %i 'contains none' rules\n",
+		printf("PP: Using %i 'contains one' rules and %i 'contains none' rules\n",
 			   rcoIDX, rcnIDX);
 	}
 }
@@ -930,7 +945,7 @@ static void prune_irrelevant_rules(Postprocessor *pp)
 Postprocessor * post_process_open(const char *path)
 {
 	Postprocessor *pp;
-	if (path==NULL) return NULL;
+	if (path == NULL) return NULL;
 
 	pp = (Postprocessor *) xalloc (sizeof(Postprocessor));
 	pp->knowledge	= pp_knowledge_open(path);
@@ -938,11 +953,11 @@ Postprocessor * post_process_open(const char *path)
 	pp->set_of_links_of_sentence = pp_linkset_open(1024);
 	pp->set_of_links_in_an_active_rule = pp_linkset_open(1024);
 	pp->relevant_contains_one_rules =
-			(int *) xalloc ((pp->knowledge->n_contains_one_rules+1)
-							*(sizeof pp->relevant_contains_one_rules[0]));
+	      (int *) xalloc ((pp->knowledge->n_contains_one_rules + 1)
+	                      *(sizeof pp->relevant_contains_one_rules[0]));
 	pp->relevant_contains_none_rules =
-			(int *) xalloc ((pp->knowledge->n_contains_none_rules+1)
-							*(sizeof pp->relevant_contains_none_rules[0]));
+	      (int *) xalloc ((pp->knowledge->n_contains_none_rules + 1)
+	                      *(sizeof pp->relevant_contains_none_rules[0]));
 	pp->relevant_contains_one_rules[0]	= -1;
 	pp->relevant_contains_none_rules[0] = -1;
 	pp->pp_node = NULL;
@@ -955,16 +970,16 @@ Postprocessor * post_process_open(const char *path)
 void post_process_close(Postprocessor *pp)
 {
 	/* frees up memory associated with pp, previously allocated by open */
-	if (pp==NULL) return;
+	if (pp == NULL) return;
 	string_set_delete(pp->sentence_link_name_set);
 	pp_linkset_close(pp->set_of_links_of_sentence);
 	pp_linkset_close(pp->set_of_links_in_an_active_rule);
 	xfree(pp->relevant_contains_one_rules,
-		(1+pp->knowledge->n_contains_one_rules)
-		*(sizeof pp->relevant_contains_one_rules[0]));
+	      (1 + pp->knowledge->n_contains_one_rules)
+	      *(sizeof pp->relevant_contains_one_rules[0]));
 	xfree(pp->relevant_contains_none_rules,
-		(1+pp->knowledge->n_contains_none_rules)
-		*(sizeof pp->relevant_contains_none_rules[0]));
+	      (1 + pp->knowledge->n_contains_none_rules)
+	      *(sizeof pp->relevant_contains_none_rules[0]));
 	pp_knowledge_close(pp->knowledge);
 	free_pp_node(pp);
 	xfree(pp, sizeof(Postprocessor));
@@ -972,7 +987,7 @@ void post_process_close(Postprocessor *pp)
 
 void post_process_close_sentence(Postprocessor *pp)
 {
-	if (pp==NULL) return;
+	if (pp == NULL) return;
 	pp_linkset_clear(pp->set_of_links_of_sentence);
 	pp_linkset_clear(pp->set_of_links_in_an_active_rule);
 	string_set_delete(pp->sentence_link_name_set);
@@ -996,15 +1011,73 @@ void post_process_scan_linkage(Postprocessor *pp, Parse_Options opts,
 	size_t i;
 	if (pp == NULL) return;
 	if (sent->length < opts->twopass_length) return;
-	for (i=0; i<sublinkage->num_links; i++)
+	for (i = 0; i < sublinkage->num_links; i++)
 	{
-		// if (sublinkage->link[i]->lw == SIZE_MAX) continue;
+		/* if (sublinkage->link[i]->lw == SIZE_MAX) continue; */
 		assert(sublinkage->link[i]->lw != SIZE_MAX);
 		p = string_set_add(sublinkage->link[i]->link_name, pp->sentence_link_name_set);
 		pp_linkset_add(pp->set_of_links_of_sentence, p);
 	}
 }
 
+static size_t report_rule_use(pp_rule *set)
+{
+	size_t cnt = 0;
+	size_t i;
+	for (i=0; set[i].msg != NULL; i++)
+	{
+		printf("usage: %d rule: %s\n", set[i].use_count, set[i].msg);
+		cnt++;
+	}
+	return cnt;
+}
+
+static size_t report_unused_rule(pp_rule *set)
+{
+	size_t i;
+	size_t cnt = 0;
+	for (i=0; set[i].msg != NULL; i++)
+	{
+		if (0 == set[i].use_count)
+		{
+			printf("Unsued rule: %s\n", set[i].msg);
+			cnt++;
+		}
+	}
+	return cnt;
+}
+
+static void report_pp_stats(Postprocessor *pp)
+{
+	size_t rule_cnt = 0;
+	size_t unused_cnt = 0;
+	pp_knowledge * kno;
+	if (verbosity < 3) return;
+
+	printf("PP stats: local_rules_firing=%d\n", pp->n_local_rules_firing);
+	kno = pp->knowledge;
+
+	printf("\nPP stats: form_a_cycle_rules\n");
+	rule_cnt += report_rule_use(kno->form_a_cycle_rules);
+
+	printf("\nPP stats: contains_one_rules\n");
+	rule_cnt += report_rule_use(kno->contains_one_rules);
+
+	printf("\nPP stats: contains_none_rules\n");
+	rule_cnt += report_rule_use(kno->contains_none_rules);
+
+	printf("\nPP stats: bounded_rules\n");
+	rule_cnt += report_rule_use(kno->bounded_rules);
+
+	printf("\nPP stats: Rules that were not used:\n");
+	unused_cnt += report_unused_rule(kno->form_a_cycle_rules);
+	unused_cnt += report_unused_rule(kno->contains_one_rules);
+	unused_cnt += report_unused_rule(kno->contains_none_rules);
+	unused_cnt += report_unused_rule(kno->bounded_rules);
+
+	printf("\nPP stats: %zd of %zd rules unused\n", unused_cnt, rule_cnt);
+}
+
 /**
  * Takes a sublinkage and returns:
  *  . for each link, the domain structure of that link
@@ -1013,7 +1086,7 @@ void post_process_scan_linkage(Postprocessor *pp, Parse_Options opts,
  * is to be ignored
  */
 PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
-							Sentence sent, Sublinkage *sublinkage, int cleanup)
+                         Sentence sent, Sublinkage *sublinkage, bool cleanup)
 {
 	const char *msg;
 
@@ -1029,9 +1102,11 @@ PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
 
 	/* The first time we see a sentence, prune the rules which we won't be
 	 * needing during postprocessing the linkages of this sentence */
-	if (sent->q_pruned_rules == FALSE && sent->length >= opts->twopass_length)
+	if (sent->q_pruned_rules == false && sent->length >= opts->twopass_length)
+	{
 		prune_irrelevant_rules(pp);
-	sent->q_pruned_rules = TRUE;
+	}
+	sent->q_pruned_rules = true;
 
 	switch (internal_process(pp, sublinkage, &msg))
 	{
@@ -1039,6 +1114,7 @@ PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
 			/* some global test failed even before we had to build the domains */
 			pp->n_global_rules_firing++;
 			pp->pp_node->violation = msg;
+			report_pp_stats(pp);
 			return pp->pp_node;
 			break;
 		case 1:
@@ -1052,6 +1128,8 @@ PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
 			break;
 	}
 
+	report_pp_stats(pp);
+
 	build_type_array(pp);
 	if (cleanup)
 	{
@@ -1115,18 +1193,18 @@ PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
   In the following sentence, then, the "Urfl_Only Domain" of the G link
   would include only the "O" link:
 
-  +-----G----+
-  +---O--+   +-AI+
-  |      |   |   |
+     +-----G----+
+     +---O--+   +-AI+
+     |      |   |   |
   hitting dogs is fun.a
 
   In the following sentence it would include the "O", the "TT", the "I",
   the second "O", and the "A".
 
-  +----------------G---------------+
-  +-----TT-----+  +-----O-----+    |
-  +---O---+    +-I+    +---A--+    +-AI+
-  |       |    |  |    |      |    |   |
+     +----------------G---------------+
+     +-----TT-----+  +-----O-----+    |
+     +---O---+    +-I+    +---A--+    +-AI+
+     |       |    |  |    |      |    |   |
   telling people to do stupid things is fun.a
 
   This would allow us to judge the following:
