@@ -16,7 +16,10 @@
 #ifdef USE_FAT_LINKAGES
 void       fat_prune(Sentence);
 bool       prune_match(int dist, Connector * left, Connector * right);
-#endif
 int        power_prune(Sentence, int mode, Parse_Options);
 void       pp_and_power_prune(Sentence, int mode, Parse_Options);
+#else
+int        power_prune(Sentence, Parse_Options);
+void       pp_and_power_prune(Sentence, Parse_Options);
+#endif /* USE_FAT_LINKAGES */
 void       expression_prune(Sentence);
