@@ -142,10 +142,10 @@ static inline const char * connector_get_string(Connector *c)
 struct Disjunct_struct
 {
     Disjunct *next;
-    const char * string;
+    const char * string;      /* subscripted dictionary word */
     Connector *left, *right;
     double cost;
-    char marked;
+    bool marked;
 };
 
 typedef struct Match_node_struct Match_node;
@@ -181,7 +181,7 @@ struct Word_struct
 	const char **alternatives;
 	X_node * x;          /* Sentence starts out with these, */
 	Disjunct * d;        /* eventually these get generated. */
-	Boolean firstupper;  /* TRUE if capitalized, and is first word of sentence */
+	bool firstupper;     /* TRUE if capitalized, and is first word of sentence */
 };
 
 
@@ -228,6 +228,14 @@ struct Andlist_struct
     int outside_word[MAX_SENTENCE];
     int cost;
 };
+
+/* These parameters tell power_pruning, to tell whether this is before
+ * or after generating and disjuncts.  GENTLE is before RUTHLESS is
+ * after.
+ */
+#define RUTHLESS 0
+#define GENTLE 1
+
 #endif /* USE_FAT_LINKAGES */
 
 /**
@@ -236,6 +244,7 @@ struct Andlist_struct
  */
 struct Linkage_info_struct
 {
+    bool discarded;
     int index;
     short N_violations;
     short unused_word_cost;
@@ -244,10 +253,10 @@ struct Linkage_info_struct
     double corpus_cost;
     size_t nwords;
 #ifdef USE_FAT_LINKAGES
-    Boolean canonical;
-    Boolean fat;
-    Boolean improper_fat_linkage;
-    Boolean inconsistent_domains;
+    bool canonical;
+    bool fat;
+    bool improper_fat_linkage;
+    bool inconsistent_domains;
     short and_cost;
     Andlist * andlist;
 #endif /* USE_FAT_LINKAGES */
@@ -266,15 +275,6 @@ struct List_o_links_struct
     List_o_links * next;
 };
 
-#ifdef USE_FAT_LINKAGES
-/* These parameters tell power_pruning, to tell whether this is before
- * or after generating and disjuncts.  GENTLE is before RUTHLESS is
- * after.
- */
-#define GENTLE 1
-#endif /* USE_FAT_LINKAGES */
-#define RUTHLESS 0
-
 typedef struct Parse_choice_struct Parse_choice;
 
 struct Parse_choice_struct
@@ -295,7 +295,7 @@ struct Parse_set_struct
 struct X_table_connector_struct
 {
     short             lw, rw;
-    unsigned short    cost;
+    unsigned short    null_count; /* number of island words */
     Parse_set         *set;
     Connector         *le, *re;
     X_table_connector *next;
@@ -304,8 +304,8 @@ struct X_table_connector_struct
 /* from string-set.c */
 struct String_set_s
 {
-    unsigned int size;       /* the current size of the table */
-    unsigned int count;      /* number of things currently in the table */
+    size_t size;       /* the current size of the table */
+    size_t count;      /* number of things currently in the table */
     char ** table;  /* the table itself */
 };
 
@@ -353,6 +353,7 @@ typedef struct pp_rule_s
     int   domain;         /* type of domain to which rule applies    */
     const char  **link_array; /* array holding the spelled-out names */
     const char  *msg;     /* explanation (NULL=end sentinel in array)*/
+    int use_count;        /* Number of times rule has been applied   */
 } pp_rule;
 
 typedef struct PPLexTable_s PPLexTable;
@@ -374,12 +375,14 @@ struct pp_knowledge_s
     pp_linkset *left_domain_starter_links;
 
     /* arrays of rules specified in knowledge file */
-    pp_rule *connected_rules, *form_a_cycle_rules;
-    pp_rule *contains_one_rules, *contains_none_rules;
+    pp_rule *form_a_cycle_rules;
+    pp_rule *contains_one_rules;
+    pp_rule *contains_none_rules;
     pp_rule *bounded_rules;
 
-    size_t n_connected_rules, n_form_a_cycle_rules;
-    size_t n_contains_one_rules, n_contains_none_rules;
+    size_t n_form_a_cycle_rules;
+    size_t n_contains_one_rules;
+    size_t n_contains_none_rules;
     size_t n_bounded_rules;
 
     size_t nStartingLinks;
