@@ -113,7 +113,7 @@ static bool is_bullet(Dictionary dict, wchar_t wc)
 	const wchar_t *bullets =
 		(const wchar_t *)AFCLASS(dict->affix_table, AFDICT_BULLETS)->string;
 
-	if (NULL == bullets) return FALSE;
+	if (NULL == bullets) return false;
 	return (NULL !=  wcschr(bullets, wc));
 }
 
@@ -261,13 +261,16 @@ void add_alternative(Sentence sent,
 				len = t_start + t_count;
 
 				/* allocate new word */
-				sent->word = (Word *)realloc(sent->word, (len+1) * sizeof(Word));
+				sent->word = realloc(sent->word, (len+1)*sizeof(Word));
 				sent->word[len].x= NULL;
 				sent->word[len].d= NULL;
 				sent->word[len].alternatives = NULL;
 				sent->word[len].unsplit_word = NULL;
 				sent->word[len].firstupper = false;
 
+				sent->post_quote = realloc(sent->post_quote, (len+1)*sizeof(bool));
+				sent->post_quote[len] = false;
+
 				t_count++;
 				if (t_count > 1) /* not first added word */
 				{
@@ -330,7 +333,7 @@ void add_alternative(Sentence sent,
 				case END:
 					assert(true, "affixtype END reached");
 			}
-			/* firstupper is false foor suffixes - start with INFIX_MARK */
+			/* firstupper is false for suffixes - start with INFIX_MARK */
 			if (is_utf8_upper(buff)) sent->word[t_start].firstupper = true;
 			lgdebug(3, " %s", ('\0' == buff[0]) ? "[empty_suffix]" : buff);
 			altappend(sent, &sent->word[t_start + ai].alternatives, buff);
@@ -348,10 +351,10 @@ void add_alternative(Sentence sent,
 
 /**
  * Return true if an alternative has been issued for the current word.
+ * t_count==0 if and only if no alternative has been issued yet.
  */
 static bool word_has_alternative(Sentence sent) {
-	if (NULL == sent->word[sent->t_start].unsplit_word) return true;
-	return false;
+	return (sent->t_count > 0);
 }
 
 /**
@@ -367,8 +370,6 @@ static bool issue_alternatives(Sentence sent,
 	if (0 == t_count) return false;
 
 	sent->word[t_start].unsplit_word = string_set_add(word, sent->string_set);
-	sent->post_quote =
-	 (bool *) realloc(sent->post_quote, (t_start+1) * sizeof(bool));
 	sent->post_quote[t_start] = quote_found;
 
 	sent->length += t_count;
@@ -536,6 +537,8 @@ static bool suffix_split(Sentence sent, const char *w, const char *wend)
 	s_strippable = suffix_list->length;
 	suffix = suffix_list->string;
 
+	if (INT_MAX == s_strippable) return false;
+
 	/* Go through once for each suffix; then go through one
 	 * final time for the no-suffix case (i.e. to look for
 	 * prefixes only, without suffixes). */
@@ -643,16 +646,16 @@ static bool mprefix_split(Sentence sent, const char *word)
 	const char **mprefix;
 	const char *newword;
 	const char *w;
-	int sz;
+	int sz = 0;
 	bool word_is_in_dict;
 	int split_prefix_i = 0;      /* split prefix index */
-	const char *split_prefix[HEB_PRENUM_MAX]; /* all prefix  */
+	const char *split_prefix[HEB_PRENUM_MAX]; /* the whole prefix */
 	/* pseen is a simple prefix combination filter */
 	bool pseen[HEB_MPREFIX_MAX]; /* prefix "subword" seen */
 	Dictionary dict = sent->dict;
 	int wordlen;
 	int wlen;
-	int plen;
+	int plen = 0;
 
 	/* set up affix table  */
 	if (NULL == dict->affix_table) return false;
@@ -783,11 +786,11 @@ static bool is_capitalizable(Sentence sent, size_t curr_word)
  * - Separating run-on words into an exact combination of words, usually 2.
  * - Find similar words.
  *
- * Return true of corrections have been issued, else false.
+ * Return true if corrections have been issued, else false.
  *
  * Note: spellcheck_suggest(), which is invoked by this function, returns
  * guesses for words containing numbers (including words consisting of digits
- * only). Hence this function should not  be called for such words.
+ * only). Hence this function should not be called for such words.
  */
 static bool guess_misspelled_word(Sentence sent, const char * word,
                                   bool quote_found, Parse_Options opts)
@@ -1116,7 +1119,7 @@ static void separate_word(Sentence sent, Parse_Options opts,
 	bool word_can_split = false;
 	bool issued = false;
 	bool try_strip_left;     /* try to strip punctuation on the left-hand side */
-	bool units_alternative = false;             /* units alternative is needed */
+	bool units_alternative = false;           /* a units alternative is needed */
 	bool parallel_regex;                  /* regex unknown word that can split */
 	bool stripped;
 	const char *wp;
@@ -1135,7 +1138,7 @@ static void separate_word(Sentence sent, Parse_Options opts,
 	size_t sz = wend - w;
 	/* Dynamic allocation of working buffers. */
 	char *word = alloca(sz+1);                   /* candidate word main buffer */
-	char *input_word = alloca(sz+1);  /* input word, possible after left-strip */
+	char *input_word = alloca(sz+1);  /* input word, possibly after left-strip */
 	int downcase_size = sz+MB_LEN_MAX+1; /* pessimistic max. size of dc buffer */
 	char *downcase = alloca(downcase_size);               /* downcasing buffer */
 	char *str = alloca(downcase_size+sizeof("[!]"));        /* tmp word buffer */
@@ -1318,7 +1321,7 @@ static void separate_word(Sentence sent, Parse_Options opts,
 			free(r_stripped_alt);
 		}
 
-		/*Restore the input word, in case it needs a further handling. */
+		/* Restore the input word, in case it needs a further handling. */
 		strcpy(word, input_word);    /* Recover the input word. */
 		n_r_stripped = 0;            /* Forget the stripping. */
 	}
@@ -1348,7 +1351,7 @@ static void separate_word(Sentence sent, Parse_Options opts,
 		free(r_stripped_alt);
 
 		/* FIXME: We suppose here that the input word cannot morpheme-split.
-		 * Anyway, the code infrastructure don't allow us to continue after
+		 * Anyway, the code infrastructure doesn't allow us to continue after
 		 * we issue alternatives for right-stripping. */
 		issue_alternatives(sent, input_word, quote_found);
 		return;
