@@ -55,7 +55,7 @@ int size_of_expression(Exp * e)
 	return size;
 }
 
-/** 
+/**
  * Build a copy of the given expression (don't copy strings, of course)
  */
 static E_list * copy_E_list(E_list * l);
@@ -126,7 +126,7 @@ static int exp_compare(Exp * e1, Exp * e2)
 }
 
 /**
- * Sub-expression matcher -- return 1 if sub is non-NULL and 
+ * Sub-expression matcher -- return 1 if sub is non-NULL and
  * contained in super, 0 otherwise.
  */
 static int exp_contains(Exp * super, Exp * sub)
@@ -135,7 +135,7 @@ static int exp_contains(Exp * super, Exp * sub)
 
 #ifdef DEBUG
 	printf("SUP: ");
-	if (super) print_expression(super); 
+	if (super) print_expression(super);
 	printf("\n");
 #endif
 
@@ -157,7 +157,7 @@ static int exp_contains(Exp * super, Exp * sub)
 
 /* ======================================================== */
 /* X_node utilities ... */
-/** 
+/**
  * frees the list of X_nodes pointed to by x, and all of the expressions
  */
 void free_X_nodes(X_node * x)
@@ -223,7 +223,7 @@ Connector * connector_new(void)
 	c->label = NORMAL_LABEL;
 	c->priority = THIN_priority;
 #endif /* USE_FAT_LINKAGES */
-	c->multi = FALSE;
+	c->multi = false;
 	c->next = NULL;
 	c->tableNext = NULL;
 	return c;
@@ -411,25 +411,25 @@ bool match_in_connector_set(Connector_set *conset, Connector * c, int dir)
  * check uses a "smart-match", the same kind used by the parser.
  */
 #if CRAZY_OBESE_CHECKING_AGLO
-Boolean word_has_connector(Dict_node * dn, const char * cs, char direction)
+bool word_has_connector(Dict_node * dn, const char * cs, char direction)
 {
 	Connector * c2 = NULL;
 	Disjunct *d, *d0;
-	if (dn == NULL) return FALSE;
+	if (dn == NULL) return false;
 	d0 = d = build_disjuncts_for_dict_node(dn);
-	if (d == NULL) return FALSE;
+	if (d == NULL) return false;
 	for (; d != NULL; d = d->next) {
 		if (direction == '+') c2 = d->right;
 		if (direction == '-') c2 = d->left;
 		for (; c2 != NULL; c2 = c2->next) {
 			if (easy_match(c2->string, cs)) {
 				free_disjuncts(d0);
-				return TRUE;
+				return true;
 			}
 		}
 	}
 	free_disjuncts(d0);
-	return FALSE;
+	return false;
 }
 #else /* CRAZY_OBESE_CHECKING_AGLO */
 
@@ -470,16 +470,16 @@ const char * word_only_connector(Dict_node * dn)
 /* ======================================================== */
 /* Dictionary utilities ... */
 
-static Boolean dn_word_contains(Dictionary dict,
-                                Dict_node * w_dn, const char * macro)
+static bool dn_word_contains(Dictionary dict,
+                             Dict_node * w_dn, const char * macro)
 {
 	Exp * m_exp;
 	Dict_node *m_dn;
 
-	if (w_dn == NULL) return FALSE;
+	if (w_dn == NULL) return false;
 
 	m_dn = dictionary_lookup_list(dict, macro);
-	if (m_dn == NULL) return FALSE;
+	if (m_dn == NULL) return false;
 
 	m_exp = m_dn->exp;
 
@@ -487,7 +487,7 @@ static Boolean dn_word_contains(Dictionary dict,
 	printf("\nWORD: ");
 	print_expression(w_dn->exp);
 	printf("\nMACR: ");
-	print_expression(m_exp); 
+	print_expression(m_exp);
 	printf("\n");
 #endif
 
@@ -496,11 +496,11 @@ static Boolean dn_word_contains(Dictionary dict,
 		if (1 == exp_contains(w_dn->exp, m_exp))
 		{
 			free_lookup_list(dict, m_dn);
-			return TRUE;
+			return true;
 		}
 	}
 	free_lookup_list(dict, m_dn);
-	return FALSE;
+	return false;
 }
 
 /**
@@ -510,14 +510,83 @@ static Boolean dn_word_contains(Dictionary dict,
  * @return: true if word's expression contains macro's expression,
  * false otherwise.
  */
-Boolean word_contains(Dictionary dict, const char * word, const char * macro)
+bool word_contains(Dictionary dict, const char * word, const char * macro)
 {
-	Dict_node *w_dn;
-	Boolean ret;
-	w_dn = abridged_lookup_list(dict, word);
-	ret = dn_word_contains(dict, w_dn, macro);
+	Dict_node *w_dn = abridged_lookup_list(dict, word);
+	bool ret = dn_word_contains(dict, w_dn, macro);
 	free_lookup_list(dict, w_dn);
 	return ret;
 }
 
+/**
+ * This hash function only looks at the leading upper case letters of
+ * the connector string, and the label fields.  This ensures that if two
+ * strings match (formally), then they must hash to the same place.
+ */
+int calculate_connector_hash(Connector * c)
+{
+	const char *s;
+	unsigned int i;
+
+	/* For most situations, all three hashes are very nearly equal;
+	 * as to which is faster depends on the parsed text.
+	 * For both English and Russian, there are about 100 pre-defined
+	 * connectors, and another 2K-4K autogen'ed ones (the IDxxx idiom
+	 * connectors, and the LLxxx suffix connectors for Russian).
+	 * Turns out the cost of settting up the hash table dominates the
+	 * cost of collistions. */
+#ifdef USE_DJB2
+	/* djb2 hash */
+	i = 5381;
+#ifdef USE_FAT_LINKAGES
+	i = ((i << 5) + i) + (0xff & c->label);
+#endif /* USE_FAT_LINKAGES */
+	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
+	while (isupper((int) *s)) /* connector tables cannot contain UTF8, yet */
+	{
+		i = ((i << 5) + i) + *s;
+		s++;
+	}
+	i += i>>14;
+#endif /* USE_DJB2 */
+
+#define USE_JENKINS
+#ifdef USE_JENKINS
+	/* Jenkins one-at-a-time hash */
+	i = 0;
+	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
+	while (isupper((int) *s)) /* connector tables cannot contain UTF8, yet */
+	{
+		i += *s;
+		i += (i<<10);
+		i ^= (i>>6);
+		s++;
+	}
+	i += (i << 3);
+	i ^= (i >> 11);
+	i += (i << 15);
+#endif /* USE_JENKINS */
+
+#ifdef USE_SDBM
+	/* sdbm hash */
+#ifdef USE_FAT_LINKAGES
+	i = (0xff & c->label);
+#else
+	i = 0;
+#endif /* USE_FAT_LINKAGES */
+	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
+	while (isupper((int) *s))
+	{
+		i = *s + (i << 6) + (i << 16) - i;
+		s++;
+	}
+#endif /* USE_SDBM */
+
+	c->hash = i;
+	return i;
+}
+
 /* ========================= END OF FILE ============================== */
